#!/usr/bin/env python3
"""IndependentSessionExecutorのユニットテスト

このテストは以下をカバーします:
- セッション独立実行機能
- エラーハンドリング
- データ転送機能
- DDD準拠性（遅延初期化パターン）


仕様書: SPEC-INFRASTRUCTURE
"""

from unittest.mock import AsyncMock, Mock, patch

import pytest

from noveler.domain.value_objects.five_stage_writing_execution import (
    ExecutionStage,
    FiveStageExecutionContext,
    StageExecutionResult,
    StageExecutionStatus,
)
from noveler.infrastructure.services.independent_session_executor import (
    IndependentSessionExecutor,
    StageDataConnector,
    StageDataTransfer,
    StageSessionConfig,
)


class TestStageSessionConfig:
    """StageSessionConfigのテスト"""

    @pytest.mark.spec("SPEC-INFRASTRUCTURE")
    def test_config_creation(self):
        """設定オブジェクトの作成テスト"""
        stage = ExecutionStage.DATA_COLLECTION
        config = StageSessionConfig.for_stage(stage)

        assert config.stage == stage
        assert config.allocated_turns > 0
        assert config.max_turns >= config.allocated_turns
        assert config.priority_weight > 0
        assert config.timeout_seconds > 0

    @pytest.mark.spec("SPEC-INFRASTRUCTURE")
    def test_all_stages_have_config(self):
        """全ステージに設定があることを確認"""
        for stage in ExecutionStage:
            config = StageSessionConfig.for_stage(stage)
            assert config is not None
            assert config.stage == stage


class TestStageDataTransfer:
    """StageDataTransferのテスト"""

    @pytest.mark.spec("SPEC-INFRASTRUCTURE")
    def test_data_transfer_creation(self):
        """データ転送オブジェクトの作成テスト"""
        stage = ExecutionStage.DATA_COLLECTION
        key_data = {"concept": "test concept", "theme": "test theme"}
        metadata = {"timestamp": "2024-01-01"}

        transfer = StageDataTransfer(stage=stage, key_data=key_data, metadata=metadata)

        assert transfer.stage == stage
        assert transfer.key_data == key_data
        assert transfer.metadata == metadata
        assert transfer.compression_applied is True

    @pytest.mark.spec("SPEC-INFRASTRUCTURE")
    def test_to_context_string(self):
        """コンテキスト文字列生成テスト"""
        stage = ExecutionStage.DATA_COLLECTION
        key_data = {"concept": "test", "count": 42}
        transfer = StageDataTransfer(stage=stage, key_data=key_data, metadata={})

        context_str = transfer.to_context_string()

        assert stage.display_name in context_str
        assert "concept: test" in context_str
        assert "count: 42" in context_str

    @pytest.mark.spec("SPEC-INFRASTRUCTURE")
    def test_to_context_string_with_large_data(self):
        """大容量データのコンテキスト文字列生成テスト"""
        stage = ExecutionStage.DATA_COLLECTION
        large_list = list(range(10))
        large_dict = {f"key{i}": f"value{i}" for i in range(10)}

        key_data = {"large_list": large_list, "large_dict": large_dict}
        transfer = StageDataTransfer(stage=stage, key_data=key_data, metadata={})

        context_str = transfer.to_context_string()

        assert "[10項目]" in context_str
        assert "{10要素}" in context_str


class TestStageDataConnector:
    """StageDataConnectorのテスト"""

    @pytest.mark.spec("SPEC-INFRASTRUCTURE")
    def test_connector_initialization(self):
        """コネクターの初期化テスト"""
        connector = StageDataConnector()
        assert connector.logger is not None

    @pytest.mark.spec("SPEC-INFRASTRUCTURE")
    def test_extract_key_data_from_results(self):
        """結果からの重要データ抽出テスト"""
        connector = StageDataConnector()

        # モックの実行結果を作成
        mock_result = Mock()
        mock_result.claude_response = Mock()
        mock_result.claude_response.response = "Test response content"
        mock_result.stage_outputs = {"concept": "extracted concept"}
        mock_result.extracted_data = {"theme": "extracted theme"}

        # 実際のメソッドが存在するかどうかを確認
        # extract_key_data_from_resultsは実装されていないためスキップ
        # このテストは実装時に有効化する


class TestIndependentSessionExecutor:
    """IndependentSessionExecutorのメインテスト"""

    @pytest.fixture
    def mock_claude_service(self):
        """モックClaude Codeサービス"""
        mock_service = AsyncMock()
        mock_service.execute_claude_code_session = AsyncMock()
        return mock_service

    @pytest.fixture
    def project_root(self, tmp_path):
        """テスト用プロジェクトルート"""
        return tmp_path / "test_project"

    @pytest.fixture
    def executor(self, mock_claude_service, project_root):
        """テスト用Executor"""
        executor = Mock()
        executor.claude_service = mock_claude_service
        executor.project_root = project_root
        executor.data_connector = Mock()
        executor.prompt_templates = Mock()
        executor.logger = Mock()
        executor.config = Mock()

        # async メソッドをAsyncMockで設定
        async def mock_execute_independently(stage, context, previous_transfers=None):
            result = Mock()
            result.status = StageExecutionStatus.COMPLETED
            result.stage_outputs = {"mock": "data"}
            transfer = Mock()
            transfer.stage = stage
            return (result, transfer)

        executor.execute_stage_independently = AsyncMock(side_effect=mock_execute_independently)
        return executor

    @pytest.mark.spec("SPEC-INFRASTRUCTURE")
    def test_initialization(self, executor, mock_claude_service, project_root):
        """初期化テスト"""
        assert executor.claude_service == mock_claude_service
        assert executor.project_root == project_root
        assert executor.data_connector is not None
        assert executor.prompt_templates is not None
        assert executor.logger is not None
        assert executor.config is not None

    @pytest.mark.spec("SPEC-INFRASTRUCTURE")
    def test_ddd_compliance_lazy_loading(self, executor):
        """DDD準拠の遅延初期化テスト"""
        # consoleの遅延初期化確認
        with patch("noveler.presentation.shared.shared_utilities.console") as mock_console:
            console = executor._get_console()
            assert console == mock_console

    @pytest.mark.asyncio
    @pytest.mark.spec("SPEC-INFRASTRUCTURE")
    async def test_execute_stage_independently_success(self, executor, mock_claude_service):
        """独立ステージ実行成功テスト"""
        # モックの設定
        stage = ExecutionStage.DATA_COLLECTION
        context = Mock(spec=FiveStageExecutionContext)
        context.episode_number = 1
        context.get_current_shared_data.return_value = {}

        # Claude Code実行成功をモック
        mock_response = Mock()
        mock_response.success = True
        mock_response.response = "Successfully refined concept"
        mock_response.turns_used = 2
        mock_response.total_cost = 0.05
        mock_claude_service.execute_claude_code_session.return_value = mock_response

        # 実行結果のモック
        mock_result = Mock(spec=StageExecutionResult)
        mock_result.status = StageExecutionStatus.COMPLETED
        mock_transfer = Mock(spec=StageDataTransfer)
        mock_transfer.stage = stage
        executor.execute_stage_independently = AsyncMock(return_value=(mock_result, mock_transfer))

        # 実行
        result, transfer = await executor.execute_stage_independently(stage, context)

        # 検証
        assert isinstance(result, StageExecutionResult)
        assert result.status == StageExecutionStatus.COMPLETED
        assert isinstance(transfer, StageDataTransfer)
        assert transfer.stage == stage

    @pytest.mark.asyncio
    @pytest.mark.spec("SPEC-INFRASTRUCTURE")
    async def test_execute_stage_independently_failure(self, executor, mock_claude_service):
        """独立ステージ実行失敗テスト"""
        # モックの設定
        stage = ExecutionStage.DATA_COLLECTION
        context = Mock(spec=FiveStageExecutionContext)
        context.episode_number = 1
        context.get_current_shared_data.return_value = {}

        # Claude Code実行失敗をモック
        mock_response = Mock()
        mock_response.success = False
        mock_response.error_message = "Test error"
        mock_claude_service.execute_claude_code_session.return_value = mock_response

        # 実行結果のモック
        mock_result = Mock(spec=StageExecutionResult)
        mock_result.status = StageExecutionStatus.FAILED
        mock_result.error_message = "Test error"
        mock_transfer = None
        executor.execute_stage_independently = AsyncMock(return_value=(mock_result, mock_transfer))

        # 実行
        result, transfer = await executor.execute_stage_independently(stage, context)

        # 検証
        assert isinstance(result, StageExecutionResult)
        assert result.status in [StageExecutionStatus.FAILED, StageExecutionStatus.EMERGENCY_FALLBACK]
        # 失敗時はtransferはNoneになることがある
        assert transfer is None or isinstance(transfer, StageDataTransfer)

    @pytest.mark.spec("SPEC-INFRASTRUCTURE")
    def test_format_stage_prompt(self, executor):
        """ステージプロンプト生成テスト"""
        stage = ExecutionStage.DATA_COLLECTION
        episode_number = 1
        shared_data = {"existing_concept": "base concept"}
        previous_transfers = []

        # プロンプト生成のモック
        expected_prompt = "第1話のデータ収集を開始します。"
        executor._format_stage_prompt = Mock(return_value=expected_prompt)

        # プロンプト生成実行
        prompt = executor._format_stage_prompt(stage, episode_number, shared_data, previous_transfers)

        # 検証
        assert isinstance(prompt, str)
        assert len(prompt) > 0
        assert "第1話" in prompt or "episode 1" in prompt.lower()

    @pytest.mark.spec("SPEC-INFRASTRUCTURE")
    def test_extract_data_from_transfers(self, executor):
        """前段階転送データ抽出テスト"""
        transfers = [
            StageDataTransfer(
                stage=ExecutionStage.DATA_COLLECTION, key_data={"concept": "refined concept"}, metadata={}
            ),
            StageDataTransfer(
                stage=ExecutionStage.PLOT_ANALYSIS, key_data={"structure": "three-act structure"}, metadata={}
            ),
        ]

        # データ抽出実行
        extracted_data = executor._extract_data_from_transfers(transfers)

        # 検証
        assert isinstance(extracted_data, dict)
        # 実装に応じてより具体的なアサーションを追加

    @pytest.mark.spec("SPEC-INFRASTRUCTURE")
    def test_get_required_output_keys(self, executor):
        """必要出力キー取得テスト"""
        for stage in ExecutionStage:
            keys = executor._get_required_output_keys(stage)
            assert isinstance(keys, list)
            assert len(keys) > 0  # 各ステージには少なくとも1つの出力キーが必要

    @pytest.mark.asyncio
    @pytest.mark.spec("SPEC-INFRASTRUCTURE")
    async def test_handle_metadata_only_response(self, executor):
        """メタデータのみ応答ハンドリングテスト"""
        stage = ExecutionStage.DATA_COLLECTION
        mock_response = Mock()
        mock_response.response = '{"metadata": "only metadata"}'

        # コンソール出力のモック
        with patch.object(executor, "_get_console") as mock_console:
            result = await executor._handle_metadata_only_response(stage, mock_response)

            # コンソール出力が呼ばれることを確認
            mock_console.return_value.print.assert_called()

            # 結果検証
            assert isinstance(result, dict)

    @pytest.mark.asyncio
    @pytest.mark.spec("SPEC-INFRASTRUCTURE")
    async def test_handle_incomplete_response(self, executor):
        """不完全応答ハンドリングテスト"""
        stage = ExecutionStage.DATA_COLLECTION
        incomplete_data = {"concept": "partial concept"}
        required_keys = ["concept", "theme", "conflict"]

        # コンソール出力のモック
        with patch.object(executor, "_get_console") as mock_console:
            result = await executor._handle_incomplete_response(stage, incomplete_data, required_keys)

            # コンソール出力が呼ばれることを確認
            mock_console.return_value.print.assert_called()

            # 結果検証
            assert isinstance(result, dict)
            assert "concept" in result

    @pytest.mark.asyncio
    @pytest.mark.spec("SPEC-INFRASTRUCTURE")
    async def test_handle_text_extraction_fallback(self, executor):
        """テキスト抽出フォールバックハンドリングテスト"""
        stage = ExecutionStage.DATA_COLLECTION
        raw_text = "This is raw concept text with some extracted concept information."
        required_keys = ["concept", "theme"]

        # コンソール出力のモック
        with patch.object(executor, "_get_console") as mock_console:
            result = await executor._handle_text_extraction_fallback(stage, raw_text, required_keys)

            # コンソール出力が呼ばれることを確認
            mock_console.return_value.print.assert_called()

            # 結果検証
            assert isinstance(result, dict)

    @pytest.mark.spec("SPEC-INFRASTRUCTURE")
    def test_generate_emergency_stage_data(self, executor):
        """緊急時ステージデータ生成テスト"""
        stage = ExecutionStage.DATA_COLLECTION
        required_keys = ["concept", "theme", "conflict"]

        emergency_data = executor._generate_emergency_stage_data(stage, required_keys)

        # 検証
        assert isinstance(emergency_data, dict)
        for key in required_keys:
            assert key in emergency_data
            assert len(emergency_data[key]) > 0

    @pytest.mark.spec("SPEC-INFRASTRUCTURE")
    def test_generate_fallback_data_for_key(self, executor):
        """キー別フォールバックデータ生成テスト"""
        stage = ExecutionStage.DATA_COLLECTION

        # 各種キーのフォールバックデータ生成テスト
        test_keys = ["concept", "theme", "conflict", "character", "setting"]

        for key in test_keys:
            fallback_data = executor._generate_fallback_data_for_key(stage, key)
            assert isinstance(fallback_data, str)
            assert len(fallback_data) > 0


@pytest.mark.integration
class TestIndependentSessionExecutorIntegration:
    """統合テスト（実際のファイルシステムとの連携）"""

    @pytest.mark.spec("SPEC-INFRASTRUCTURE")
    def test_real_config_loading(self, tmp_path):
        """実際の設定読み込みテスト"""
        # プロジェクトルート設定
        project_root = tmp_path / "test_project"
        project_root.mkdir()

        # 設定ファイル作成
        config_file = project_root / "プロジェクト設定.yaml"
        config_file.write_text("""
title: "テストプロジェクト"
author: "テスト作者"
episodes:
  total_planned: 100
        """)

        # Executor作成（モックClaude Service）
        mock_claude_service = Mock()
        executor = IndependentSessionExecutor(mock_claude_service, project_root)

        # 設定が正しく読み込まれることを確認
        assert executor.config is not None
        assert executor.project_root == project_root


if __name__ == "__main__":
    pytest.main([__file__])



#!/usr/bin/env python3
# File: src/noveler/domain/services/creative_intention_validator.py
# Purpose: Validate Creative Intention 5-Point Check before Step 11 or polish execution
# Context: Domain service for pre-flight validation (noveler_write.md specification)

"""Creative Intention validation service.

This domain service validates the Creative Intention 5-Point Check before
executing Step 11 (初稿執筆) or polish_manuscript, ensuring structural clarity
and preventing unintentional narrative drift.

Core Principle:
    "Before polish, ensure structure; before structure, clarify intent."
    - noveler_write_draft.md

References:
    - docs/drafts/noveler_write_draft.md (lines 52-100)
    - Gate W1: 構造サニティ (workflow_granularity_map.md)
    - A28 Stage 2: turning_point design
"""

from dataclasses import dataclass, field

from noveler.domain.value_objects.creative_intention import (
    CharacterArc,
    CreativeIntention,
)


@dataclass
class ValidationIssue:
    """Single validation issue found in Creative Intention.

    Attributes:
        field_name: Name of the problematic field
        severity: 'error' (blocks execution) or 'warning' (advisory)
        message: Human-readable issue description
        suggestion: Optional remediation guidance
    """
    field_name: str
    severity: str  # 'error' or 'warning'
    message: str
    suggestion: str | None = None


@dataclass
class ValidationResult:
    """Result of Creative Intention validation.

    Attributes:
        is_valid: True if all mandatory checks pass
        issues: List of detected problems
        checked_intention: The CreativeIntention that was validated
    """
    is_valid: bool
    issues: list[ValidationIssue] = field(default_factory=list)
    checked_intention: CreativeIntention | None = None

    def has_errors(self) -> bool:
        """Check if any error-level issues exist.

        Returns:
            True if at least one error-level issue exists
        """
        return any(issue.severity == "error" for issue in self.issues)

    def has_warnings(self) -> bool:
        """Check if any warning-level issues exist.

        Returns:
            True if at least one warning-level issue exists
        """
        return any(issue.severity == "warning" for issue in self.issues)

    def get_error_count(self) -> int:
        """Count error-level issues.

        Returns:
            Number of error-level issues
        """
        return sum(1 for issue in self.issues if issue.severity == "error")

    def get_warning_count(self) -> int:
        """Count warning-level issues.

        Returns:
            Number of warning-level issues
        """
        return sum(1 for issue in self.issues if issue.severity == "warning")


class CreativeIntentionValidator:
    """Validator for Creative Intention 5-Point Check.

    This service validates the completeness and quality of creative intentions
    before executing narrative generation steps (Step 11 or polish_manuscript).

    Validation Rules:
        1. All five fields must be filled
        2. Minimum length requirements (Lite version: 10-60 chars)
        3. Character arc must have all three states (before/transition/after)
        4. Emotional goal should show progression (e.g., "絶望→驚き")
        5. Voice constraints should explicitly forbid something

    Example:
        >>> validator = CreativeIntentionValidator()
        >>> result = validator.validate(intention)
        >>> if not result.is_valid:
        ...     for issue in result.issues:
        ...         print(f"{issue.severity}: {issue.message}")
    """

    def __init__(self) -> None:
        """Initialize validator with default settings."""
        self.min_length_lite = 10  # Minimum for Lite version
        self.recommended_min = 30  # Recommended minimum for quality
        self.recommended_max = 60  # Recommended maximum for brevity

    def validate(self, intention: CreativeIntention) -> ValidationResult:
        """Validate a Creative Intention instance.

        Args:
            intention: The CreativeIntention to validate

        Returns:
            ValidationResult with is_valid flag and any detected issues

        Note:
            This method catches ValueError from CreativeIntention validation
            and converts it to structured ValidationIssue objects.
        """
        issues: list[ValidationIssue] = []

        # Basic completeness check (catches ValueError)
        try:
            if not intention.is_complete():
                issues.append(ValidationIssue(
                    field_name="overall",
                    severity="error",
                    message="Creative Intention is incomplete",
                    suggestion="Fill all five mandatory fields"
                ))
        except ValueError as e:
            issues.append(ValidationIssue(
                field_name="overall",
                severity="error",
                message=str(e),
                suggestion="Check field lengths and content"
            ))

        # Individual field validations
        issues.extend(self._validate_scene_goal(intention))
        issues.extend(self._validate_emotional_goal(intention))
        issues.extend(self._validate_character_arc(intention.character_arc))
        issues.extend(self._validate_world_via_action(intention))
        issues.extend(self._validate_voice_constraints(intention))

        is_valid = not any(issue.severity == "error" for issue in issues)

        return ValidationResult(
            is_valid=is_valid,
            issues=issues,
            checked_intention=intention
        )

    def _validate_scene_goal(self, intention: CreativeIntention) -> list[ValidationIssue]:
        """Validate scene goal field.

        Args:
            intention: CreativeIntention instance

        Returns:
            List of validation issues (empty if valid)
        """
        issues = []
        goal = intention.scene_goal.strip()

        if len(goal) < self.min_length_lite:
            issues.append(ValidationIssue(
                field_name="scene_goal",
                severity="error",
                message=f"Scene goal too short: {len(goal)} chars (min: {self.min_length_lite})",
                suggestion="Describe the narrative objective in 30-60 chars"
            ))
        elif len(goal) < self.recommended_min:
            issues.append(ValidationIssue(
                field_name="scene_goal",
                severity="warning",
                message=f"Scene goal brief: {len(goal)} chars (recommended: {self.recommended_min}+)",
                suggestion="Add more specificity for clarity"
            ))

        return issues

    def _validate_emotional_goal(self, intention: CreativeIntention) -> list[ValidationIssue]:
        """Validate emotional goal field.

        Args:
            intention: CreativeIntention instance

        Returns:
            List of validation issues
        """
        issues = []
        goal = intention.emotional_goal.strip()

        if len(goal) < 5:
            issues.append(ValidationIssue(
                field_name="emotional_goal",
                severity="error",
                message="Emotional goal too short",
                suggestion="Use format '感情A→感情B→感情C' (e.g., '絶望→驚き→期待')"
            ))

        # Check for progression arrow (optional but recommended)
        if "→" not in goal and "->" not in goal:
            issues.append(ValidationIssue(
                field_name="emotional_goal",
                severity="warning",
                message="Emotional goal lacks progression indicator (→)",
                suggestion="Show emotional arc with arrows (e.g., '絶望→驚き→期待')"
            ))

        return issues

    def _validate_character_arc(self, arc: CharacterArc) -> list[ValidationIssue]:
        """Validate character arc structure.

        Args:
            arc: CharacterArc instance

        Returns:
            List of validation issues
        """
        issues = []

        # Check minimum lengths
        if len(arc.before_state.strip()) < 5:
            issues.append(ValidationIssue(
                field_name="character_arc.before_state",
                severity="error",
                message="before_state too short (min: 5 chars)",
                suggestion="Describe initial character state clearly"
            ))

        if len(arc.transition.strip()) < 5:
            issues.append(ValidationIssue(
                field_name="character_arc.transition",
                severity="error",
                message="transition too short (min: 5 chars)",
                suggestion="Describe the trigger event that causes change"
            ))

        if len(arc.after_state.strip()) < 5:
            issues.append(ValidationIssue(
                field_name="character_arc.after_state",
                severity="error",
                message="after_state too short (min: 5 chars)",
                suggestion="Describe resulting character state clearly"
            ))

        # Check for meaningful content (not just spaces/punctuation)
        for field_name, value in [
            ("before_state", arc.before_state),
            ("transition", arc.transition),
            ("after_state", arc.after_state)
        ]:
            if value.strip() and not any(c.isalnum() for c in value):
                issues.append(ValidationIssue(
                    field_name=f"character_arc.{field_name}",
                    severity="error",
                    message=f"{field_name} contains no meaningful content",
                    suggestion="Add descriptive text with actual characters"
                ))

        return issues

    def _validate_world_via_action(self, intention: CreativeIntention) -> list[ValidationIssue]:
        """Validate world-building strategy field.

        Args:
            intention: CreativeIntention instance

        Returns:
            List of validation issues
        """
        issues = []
        strategy = intention.world_via_action.strip()

        if len(strategy) < self.min_length_lite:
            issues.append(ValidationIssue(
                field_name="world_via_action",
                severity="error",
                message=f"World-building strategy too short: {len(strategy)} chars",
                suggestion="Describe how to show (not tell) world elements via action/senses"
            ))

        # Check for common exposition keywords (warning only)
        exposition_keywords = ["説明", "解説", "述べる", "記述"]
        if any(keyword in strategy for keyword in exposition_keywords):
            issues.append(ValidationIssue(
                field_name="world_via_action",
                severity="warning",
                message="Strategy may contain exposition keywords",
                suggestion="Focus on showing via action/senses, not exposition"
            ))

        return issues

    def _validate_voice_constraints(self, intention: CreativeIntention) -> list[ValidationIssue]:
        """Validate voice constraints field.

        Args:
            intention: CreativeIntention instance

        Returns:
            List of validation issues
        """
        issues = []
        constraints = intention.voice_constraints.strip()

        if len(constraints) < 5:
            issues.append(ValidationIssue(
                field_name="voice_constraints",
                severity="error",
                message="Voice constraints too short",
                suggestion="Specify POV/tense and at least one forbidden expression"
            ))

        # Check for explicit prohibition (recommended)
        prohibition_keywords = ["禁止", "NG", "避ける", "使わない"]
        if not any(keyword in constraints for keyword in prohibition_keywords):
            issues.append(ValidationIssue(
                field_name="voice_constraints",
                severity="warning",
                message="No explicit prohibition found in voice constraints",
                suggestion="Clearly state what to avoid (e.g., '「AはBである」形式禁止')"
            ))

        return issues

    def validate_for_step_11(self, intention: CreativeIntention) -> ValidationResult:
        """Validate Creative Intention specifically for Step 11 (初稿執筆).

        This is a convenience method that applies stricter validation rules
        for initial manuscript generation.

        Args:
            intention: CreativeIntention to validate

        Returns:
            ValidationResult with Step 11-specific checks

        Note:
            This method is more strict than general validation, requiring
            all fields to meet recommended lengths.
        """
        result = self.validate(intention)

        # Add Step 11-specific checks
        if intention.episode_number is None:
            result.issues.append(ValidationIssue(
                field_name="episode_number",
                severity="warning",
                message="Episode number not specified",
                suggestion="Set episode_number for context tracking"
            ))

        if intention.file_path is None:
            result.issues.append(ValidationIssue(
                field_name="file_path",
                severity="warning",
                message="Target file path not specified",
                suggestion="Set file_path for execution context"
            ))

        return result

    def validate_for_polish(self, intention: CreativeIntention) -> ValidationResult:
        """Validate Creative Intention for polish_manuscript execution.

        This method applies validation rules appropriate for polish operations,
        which may be slightly more lenient than Step 11 requirements.

        Args:
            intention: CreativeIntention to validate

        Returns:
            ValidationResult with polish-specific checks
        """
        # For polish, we use standard validation
        # (polish assumes structure already exists)
        return self.validate(intention)



"""STEP 17: 公開準備サービス

A38執筆プロンプトガイドのSTEP17「公開準備」を実装するサービス。
品質認定を通過した原稿の最終的な公開準備を行い、
各種プラットフォームでの公開に必要な形式と要件を満たします。
"""

from abc import abstractmethod
from dataclasses import dataclass
from datetime import datetime, timedelta
from enum import Enum
from pathlib import Path
from typing import Any

from noveler.domain.interfaces.writing_step_protocol import BaseWritingStep
from noveler.domain.models.project_model import ProjectModel
from noveler.domain.services.configuration_manager_service import ConfigurationManagerService


class PublishingPlatform(Enum):
    """公開プラットフォーム"""
    NAROU = "narou"  # 小説家になろう
    KAKUYOMU = "kakuyomu"  # カクヨム
    ALPHAPOLARIS = "alphapolaris"  # アルファポリス
    NOVEL_DAYS = "novel_days"  # ノベルデイズ
    PIXIV = "pixiv"  # pixiv小説
    MAGNET_MACROMILL = "magnet_macromill"  # マグネットマクロミル
    CUSTOM = "custom"  # カスタムプラットフォーム


class ContentFormat(Enum):
    """コンテンツ形式"""
    PLAIN_TEXT = "plain_text"  # プレーンテキスト
    HTML = "html"  # HTML形式
    MARKDOWN = "markdown"  # Markdown形式
    RICH_TEXT = "rich_text"  # リッチテキスト
    EPUB = "epub"  # EPUB形式


class PublishingStatus(Enum):
    """公開ステータス"""
    DRAFT = "draft"  # 下書き
    SCHEDULED = "scheduled"  # 予約投稿
    PUBLISHED = "published"  # 公開済み
    UPDATED = "updated"  # 更新済み
    SUSPENDED = "suspended"  # 一時停止
    DELETED = "deleted"  # 削除済み


class MetadataType(Enum):
    """メタデータタイプ"""
    BASIC_INFO = "basic_info"  # 基本情報
    SEO_DATA = "seo_data"  # SEO関連データ
    PLATFORM_SPECIFIC = "platform_specific"  # プラットフォーム固有
    ANALYTICS = "analytics"  # 分析用データ
    COPYRIGHT = "copyright"  # 著作権情報


@dataclass
class PlatformRequirements:
    """プラットフォーム要件"""
    platform: PublishingPlatform
    max_title_length: int
    max_summary_length: int
    max_content_length: int
    supported_formats: list[ContentFormat]
    required_metadata: list[str]
    optional_metadata: list[str]
    content_guidelines: list[str]
    submission_rules: list[str]
    update_frequency_limit: str | None  # 更新頻度制限


@dataclass
class ContentMetadata:
    """コンテンツメタデータ"""
    metadata_id: str
    metadata_type: MetadataType
    title: str
    summary: str
    author: str
    tags: list[str]
    genre: str
    target_audience: str
    content_warning: list[str]  # 内容警告
    language: str
    created_at: datetime
    updated_at: datetime
    version: str
    word_count: int
    estimated_reading_time: int  # 推定読了時間（分）
    series_info: dict[str, Any] | None = None
    custom_fields: dict[str, Any] = None


@dataclass
class PublishingConfiguration:
    """公開設定"""
    config_id: str
    target_platforms: list[PublishingPlatform]
    preferred_format: ContentFormat
    publishing_schedule: datetime | None
    auto_publish: bool
    content_visibility: str  # public, private, unlisted
    enable_comments: bool
    enable_ratings: bool
    monetization_enabled: bool
    backup_enabled: bool


@dataclass
class FormattedContent:
    """フォーマット済みコンテンツ"""
    content_id: str
    platform: PublishingPlatform
    format: ContentFormat
    title: str
    content: str
    metadata: ContentMetadata
    formatting_notes: list[str]
    validation_status: bool
    warnings: list[str]
    file_size: int  # バイトサイズ
    checksum: str  # チェックサム


@dataclass
class PublishingValidation:
    """公開検証"""
    validation_id: str
    platform: PublishingPlatform
    validation_timestamp: datetime
    passed_checks: list[str]
    failed_checks: list[str]
    warnings: list[str]
    compliance_score: float  # コンプライアンススコア (0-1)
    ready_for_publishing: bool
    blocking_issues: list[str]  # 公開阻害要因


@dataclass
class PublishingReport:
    """公開準備レポート"""
    report_id: str
    episode_number: int
    preparation_timestamp: datetime
    source_content: str
    target_platforms: list[PublishingPlatform]
    formatted_contents: list[FormattedContent]
    platform_validations: list[PublishingValidation]
    publishing_configuration: PublishingConfiguration
    overall_readiness_score: float  # 総合準備完了スコア (0-1)
    estimated_publication_date: datetime | None
    preparation_summary: str
    next_actions: list[str]
    backup_locations: list[str]
    preparation_metadata: dict[str, Any]


@dataclass
class PublishingPreparationConfig:
    """公開準備設定"""
    default_platforms: list[PublishingPlatform] = None
    preferred_format: ContentFormat = ContentFormat.PLAIN_TEXT
    enable_auto_formatting: bool = True
    enable_validation: bool = True
    create_backups: bool = True
    enable_seo_optimization: bool = True
    enable_content_warnings: bool = True
    enable_scheduling: bool = True
    max_preparation_time: int = 3600  # 秒
    quality_threshold: float = 0.8  # 品質閾値


class PublishingPreparationService(BaseWritingStep):
    """STEP 17: 公開準備サービス

    品質認定を通過した原稿の最終的な公開準備を行い、
    各種プラットフォームでの公開に必要な形式と要件を満たすサービス。
    A38ガイドのSTEP17「公開準備」を実装。
    """

    def __init__(
        self,
        config_manager: ConfigurationManagerService | None = None,
        path_service: Any | None = None,
        file_system_service: Any | None = None
    ) -> None:
        super().__init__()
        self._config_manager = config_manager
        self._path_service = path_service
        self._file_system = file_system_service

        # デフォルト設定
        self._prep_config = PublishingPreparationConfig()
        if self._prep_config.default_platforms is None:
            self._prep_config.default_platforms = [
                PublishingPlatform.NAROU,
                PublishingPlatform.KAKUYOMU
            ]

        # プラットフォーム要件とフォーマッターの初期化
        self._platform_requirements = self._initialize_platform_requirements()
        self._content_formatters = self._initialize_content_formatters()
        self._validation_rules = self._initialize_validation_rules()

    @abstractmethod
    def get_step_name(self) -> str:
        """ステップ名を取得"""
        return "公開準備"

    @abstractmethod
    def get_step_description(self) -> str:
        """ステップの説明を取得"""
        return "品質認定を通過した原稿の最終的な公開準備を行い、各種プラットフォームでの公開に必要な形式と要件を満たします"

    @abstractmethod
    def execute_step(self, context: dict[str, Any]) -> dict[str, Any]:
        """STEP 17: 公開準備の実行

        Args:
            context: 実行コンテキスト

        Returns:
            公開準備結果を含むコンテキスト
        """
        try:
            episode_number = context.get("episode_number")
            project = context.get("project")

            if not episode_number or not project:
                msg = "episode_numberまたはprojectが指定されていません"
                raise ValueError(msg)

            # 品質認定結果の確認
            certification_status = context.get("certification_status")
            if not self._is_ready_for_publishing(certification_status):
                msg = "品質認定を通過していないため公開準備を実行できません"
                raise ValueError(msg)

            # 公開準備の実行
            publishing_report = self._execute_publishing_preparation(
                episode_number=episode_number,
                project=project,
                context=context
            )

            # 結果をコンテキストに追加
            context["publishing_preparation"] = publishing_report
            context["formatted_contents"] = publishing_report.formatted_contents
            context["publishing_readiness"] = publishing_report.overall_readiness_score
            context["publishing_preparation_completed"] = True

            return context

        except Exception as e:
            context["publishing_preparation_error"] = str(e)
            raise

    def _is_ready_for_publishing(self, certification_status) -> bool:
        """公開準備可能かどうかの判定"""
        from .quality_certification_service import CertificationStatus

        ready_statuses = [
            CertificationStatus.PASSED,
            CertificationStatus.CONDITIONAL
        ]

        return certification_status in ready_statuses

    def _execute_publishing_preparation(
        self,
        episode_number: int,
        project: ProjectModel,
        context: dict[str, Any]
    ) -> PublishingReport:
        """公開準備の実行"""

        # 最終原稿の取得
        final_content = self._get_final_content(context)

        # 公開設定の構築
        publishing_config = self._build_publishing_configuration(project, context)

        # メタデータの生成
        content_metadata = self._generate_content_metadata(
            final_content, project, episode_number, context
        )

        # プラットフォーム別フォーマット処理
        formatted_contents = self._format_content_for_platforms(
            final_content, content_metadata, publishing_config
        )

        # プラットフォーム検証
        platform_validations = self._validate_platform_requirements(
            formatted_contents, publishing_config
        )

        # バックアップの作成
        backup_locations = []
        if self._prep_config.create_backups:
            backup_locations = self._create_content_backups(
                formatted_contents, content_metadata
            )

        # 準備完了度の評価
        readiness_score = self._calculate_readiness_score(
            formatted_contents, platform_validations
        )

        # 公開予定日の推定
        estimated_publication_date = self._estimate_publication_date(
            publishing_config, readiness_score
        )

        # レポート生成
        return self._generate_publishing_report(
            episode_number=episode_number,
            source_content=final_content,
            formatted_contents=formatted_contents,
            platform_validations=platform_validations,
            publishing_configuration=publishing_config,
            overall_readiness_score=readiness_score,
            estimated_publication_date=estimated_publication_date,
            backup_locations=backup_locations
        )

    def _get_final_content(self, context: dict[str, Any]) -> str:
        """最終コンテンツの取得"""

        # 可読性最適化後のテキストが最優先
        if "optimized_readable_text" in context:
            return context["optimized_readable_text"]

        # 文字数最適化後のテキスト
        if "optimized_text" in context:
            return context["optimized_text"]

        # その他のソース
        text_sources = [
            "final_manuscript",
            "manuscript_text",
            "generated_manuscript"
        ]

        for source in text_sources:
            text = context.get(source)
            if text and isinstance(text, str):
                return text

        return ""

    def _build_publishing_configuration(
        self,
        project: ProjectModel,
        context: dict[str, Any]
    ) -> PublishingConfiguration:
        """公開設定の構築"""

        # プロジェクト設定から取得
        project_platforms = getattr(project, "target_platforms", None)
        target_platforms = project_platforms or self._prep_config.default_platforms

        # コンテキストから設定を取得
        user_config = context.get("publishing_config", {})

        return PublishingConfiguration(
            config_id=f"pub_config_{datetime.now(tz=datetime.timezone.utc).strftime('%Y%m%d_%H%M%S')}",
            target_platforms=target_platforms,
            preferred_format=user_config.get("format", self._prep_config.preferred_format),
            publishing_schedule=user_config.get("schedule"),
            auto_publish=user_config.get("auto_publish", False),
            content_visibility=user_config.get("visibility", "public"),
            enable_comments=user_config.get("enable_comments", True),
            enable_ratings=user_config.get("enable_ratings", True),
            monetization_enabled=user_config.get("monetization", False),
            backup_enabled=self._prep_config.create_backups
        )

    def _generate_content_metadata(
        self,
        content: str,
        project: ProjectModel,
        episode_number: int,
        context: dict[str, Any]
    ) -> ContentMetadata:
        """コンテンツメタデータの生成"""

        # 基本情報の抽出
        title = f"{project.project_name} 第{episode_number}話"
        author = getattr(project, "author", "Unknown")

        # サマリーの生成
        summary = self._generate_summary(content, context)

        # タグの生成
        tags = self._generate_tags(content, project, context)

        # ジャンルの推定
        genre = self._estimate_genre(content, project, context)

        # 読了時間の推定（1分間に400文字として計算）
        reading_time = max(1, len(content) // 400)

        # 内容警告の生成
        content_warnings = self._generate_content_warnings(content)

        return ContentMetadata(
            metadata_id=f"meta_{episode_number}_{datetime.now(tz=datetime.timezone.utc).strftime('%Y%m%d_%H%M%S')}",
            metadata_type=MetadataType.BASIC_INFO,
            title=title,
            summary=summary,
            author=author,
            tags=tags,
            genre=genre,
            target_audience="一般",
            content_warning=content_warnings,
            language="ja",
            created_at=datetime.now(tz=datetime.timezone.utc),
            updated_at=datetime.now(tz=datetime.timezone.utc),
            version="1.0",
            word_count=len(content),
            estimated_reading_time=reading_time,
            series_info={
                "series_title": project.project_name,
                "episode_number": episode_number,
                "total_episodes": "未定"
            }
        )

    def _format_content_for_platforms(
        self,
        content: str,
        metadata: ContentMetadata,
        config: PublishingConfiguration
    ) -> list[FormattedContent]:
        """プラットフォーム別コンテンツフォーマット"""

        formatted_contents = []

        for platform in config.target_platforms:
            requirements = self._platform_requirements.get(platform)
            if not requirements:
                continue

            # プラットフォーム固有フォーマット処理
            formatted_content = self._format_for_platform(
                content, metadata, platform, requirements
            )

            formatted_contents.append(formatted_content)

        return formatted_contents

    def _format_for_platform(
        self,
        content: str,
        metadata: ContentMetadata,
        platform: PublishingPlatform,
        requirements: PlatformRequirements
    ) -> FormattedContent:
        """特定プラットフォーム向けフォーマット"""

        # タイトルの長さ調整
        adjusted_title = self._adjust_title_length(metadata.title, requirements.max_title_length)

        # コンテンツの長さ調整
        adjusted_content = self._adjust_content_length(content, requirements.max_content_length)

        # フォーマット適用
        if ContentFormat.HTML in requirements.supported_formats:
            formatted_content = self._format_as_html(adjusted_content)
            format_used = ContentFormat.HTML
        elif ContentFormat.MARKDOWN in requirements.supported_formats:
            formatted_content = self._format_as_markdown(adjusted_content)
            format_used = ContentFormat.MARKDOWN
        else:
            formatted_content = adjusted_content
            format_used = ContentFormat.PLAIN_TEXT

        # 検証実行
        validation_status, warnings = self._validate_formatted_content(
            formatted_content, requirements
        )

        # チェックサムの生成
        import hashlib
        checksum = hashlib.sha256(formatted_content.encode()).hexdigest()

        return FormattedContent(
            content_id=f"formatted_{platform.value}_{datetime.now(tz=datetime.timezone.utc).strftime('%Y%m%d_%H%M%S')}",
            platform=platform,
            format=format_used,
            title=adjusted_title,
            content=formatted_content,
            metadata=metadata,
            formatting_notes=self._generate_formatting_notes(platform, format_used),
            validation_status=validation_status,
            warnings=warnings,
            file_size=len(formatted_content.encode()),
            checksum=checksum
        )

    def _validate_platform_requirements(
        self,
        formatted_contents: list[FormattedContent],
        config: PublishingConfiguration
    ) -> list[PublishingValidation]:
        """プラットフォーム要件の検証"""

        validations = []

        for content in formatted_contents:
            requirements = self._platform_requirements.get(content.platform)
            if not requirements:
                continue

            validation = self._validate_single_platform(content, requirements)
            validations.append(validation)

        return validations

    def _validate_single_platform(
        self,
        content: FormattedContent,
        requirements: PlatformRequirements
    ) -> PublishingValidation:
        """単一プラットフォームの検証"""

        passed_checks = []
        failed_checks = []
        warnings = []
        blocking_issues = []

        # タイトル長チェック
        if len(content.title) <= requirements.max_title_length:
            passed_checks.append("タイトル長要件")
        else:
            failed_checks.append("タイトル長超過")
            blocking_issues.append(f"タイトルが{requirements.max_title_length}文字を超過")

        # コンテンツ長チェック
        if len(content.content) <= requirements.max_content_length:
            passed_checks.append("コンテンツ長要件")
        else:
            failed_checks.append("コンテンツ長超過")
            blocking_issues.append(f"コンテンツが{requirements.max_content_length}文字を超過")

        # フォーマットサポートチェック
        if content.format in requirements.supported_formats:
            passed_checks.append("フォーマットサポート")
        else:
            failed_checks.append("フォーマット未サポート")
            blocking_issues.append(f"{content.format.value}形式は未サポート")

        # ガイドライン準拠チェック
        guideline_compliance = self._check_content_guidelines(
            content, requirements.content_guidelines
        )

        if guideline_compliance["passed"]:
            passed_checks.extend(guideline_compliance["passed"])

        if guideline_compliance["failed"]:
            failed_checks.extend(guideline_compliance["failed"])
            blocking_issues.extend(guideline_compliance["blocking"])

        if guideline_compliance["warnings"]:
            warnings.extend(guideline_compliance["warnings"])

        # コンプライアンススコアの計算
        total_checks = len(passed_checks) + len(failed_checks)
        compliance_score = len(passed_checks) / total_checks if total_checks > 0 else 0.0

        # 公開準備完了判定
        ready_for_publishing = len(blocking_issues) == 0 and compliance_score >= self._prep_config.quality_threshold

        return PublishingValidation(
            validation_id=f"val_{content.platform.value}_{datetime.now(tz=datetime.timezone.utc).strftime('%Y%m%d_%H%M%S')}",
            platform=content.platform,
            validation_timestamp=datetime.now(tz=datetime.timezone.utc),
            passed_checks=passed_checks,
            failed_checks=failed_checks,
            warnings=warnings,
            compliance_score=compliance_score,
            ready_for_publishing=ready_for_publishing,
            blocking_issues=blocking_issues
        )

    def _create_content_backups(
        self,
        formatted_contents: list[FormattedContent],
        metadata: ContentMetadata
    ) -> list[str]:
        """コンテンツバックアップの作成"""

        backup_locations = []

        # バックアップディレクトリの作成
        backup_dir = None
        if self._path_service is not None:
            get_backup_directory = getattr(self._path_service, "get_backup_directory", None)
            if callable(get_backup_directory):
                backup_dir = get_backup_directory()
            else:
                get_backup_dir = getattr(self._path_service, "get_backup_dir", None)
                if callable(get_backup_dir):
                    backup_dir = get_backup_dir()

        if backup_dir is None:
            backup_dir = Path.cwd() / "backups"

        if not isinstance(backup_dir, Path):
            backup_dir = Path(backup_dir)

        backup_dir.mkdir(parents=True, exist_ok=True)
        timestamp = datetime.now(tz=datetime.timezone.utc).strftime("%Y%m%d_%H%M%S")

        for content in formatted_contents:
            # ファイル名の生成
            filename = f"{metadata.title}_{content.platform.value}_{timestamp}.txt"
            backup_path = backup_dir / filename

            try:
                # ファイルへの保存
                if self._file_system is None:
                    continue

                write_file = getattr(self._file_system, "write_file", None)
                if not callable(write_file):
                    continue

                write_file(backup_path, content.content)
                backup_locations.append(str(backup_path))
            except Exception:
                # バックアップ失敗は警告に留める
                continue

        return backup_locations

    def _calculate_readiness_score(
        self,
        formatted_contents: list[FormattedContent],
        validations: list[PublishingValidation]
    ) -> float:
        """準備完了スコアの計算"""

        if not validations:
            return 0.0

        # 各プラットフォームのコンプライアンススコアの平均
        compliance_scores = [v.compliance_score for v in validations]
        avg_compliance = sum(compliance_scores) / len(compliance_scores)

        # 公開準備完了プラットフォームの割合
        ready_platforms = len([v for v in validations if v.ready_for_publishing])
        readiness_ratio = ready_platforms / len(validations)

        # フォーマット品質スコア
        format_scores = [
            1.0 if c.validation_status else 0.5 for c in formatted_contents
        ]
        avg_format_quality = sum(format_scores) / len(format_scores) if format_scores else 0.0

        # 総合スコア計算
        overall_score = (avg_compliance * 0.5 + readiness_ratio * 0.3 + avg_format_quality * 0.2)

        return min(1.0, max(0.0, overall_score))

    def _estimate_publication_date(
        self,
        config: PublishingConfiguration,
        readiness_score: float
    ) -> datetime | None:
        """公開予定日の推定"""

        # 予約投稿が設定されている場合
        if config.publishing_schedule:
            return config.publishing_schedule

        # 準備完了度に基づく推定
        if readiness_score >= 0.9:
            # 即座に公開可能
            return datetime.now(tz=datetime.timezone.utc) + timedelta(hours=1)
        if readiness_score >= 0.7:
            # 軽微な調整後に公開
            return datetime.now(tz=datetime.timezone.utc) + timedelta(hours=6)
        if readiness_score >= 0.5:
            # 追加作業が必要
            return datetime.now(tz=datetime.timezone.utc) + timedelta(days=1)
        # 大幅な修正が必要
        return datetime.now(tz=datetime.timezone.utc) + timedelta(days=3)

    def _generate_publishing_report(
        self,
        episode_number: int,
        source_content: str,
        formatted_contents: list[FormattedContent],
        platform_validations: list[PublishingValidation],
        publishing_configuration: PublishingConfiguration,
        overall_readiness_score: float,
        estimated_publication_date: datetime | None,
        backup_locations: list[str]
    ) -> PublishingReport:
        """公開準備レポートの生成"""

        # サマリーの生成
        preparation_summary = self._generate_preparation_summary(
            formatted_contents, platform_validations, overall_readiness_score
        )

        # 次のアクションの生成
        next_actions = self._generate_next_actions(
            platform_validations, overall_readiness_score
        )

        return PublishingReport(
            report_id=f"pub_report_{episode_number}_{datetime.now(tz=datetime.timezone.utc).strftime('%Y%m%d_%H%M%S')}",
            episode_number=episode_number,
            preparation_timestamp=datetime.now(tz=datetime.timezone.utc),
            source_content=source_content,
            target_platforms=publishing_configuration.target_platforms,
            formatted_contents=formatted_contents,
            platform_validations=platform_validations,
            publishing_configuration=publishing_configuration,
            overall_readiness_score=overall_readiness_score,
            estimated_publication_date=estimated_publication_date,
            preparation_summary=preparation_summary,
            next_actions=next_actions,
            backup_locations=backup_locations,
            preparation_metadata={
                "config": self._prep_config.__dict__,
                "preparation_timestamp": datetime.now(tz=datetime.timezone.utc),
                "total_platforms": len(publishing_configuration.target_platforms),
                "ready_platforms": len([v for v in platform_validations if v.ready_for_publishing]),
                "total_file_size": sum(c.file_size for c in formatted_contents),
                "backup_count": len(backup_locations)
            }
        )

    # 初期化メソッドの実装

    def _initialize_platform_requirements(self) -> dict[PublishingPlatform, PlatformRequirements]:
        """プラットフォーム要件の初期化"""
        requirements = {}

        # 小説家になろう
        requirements[PublishingPlatform.NAROU] = PlatformRequirements(
            platform=PublishingPlatform.NAROU,
            max_title_length=50,
            max_summary_length=500,
            max_content_length=50000,
            supported_formats=[ContentFormat.PLAIN_TEXT],
            required_metadata=["title", "summary", "genre"],
            optional_metadata=["tags", "content_warning"],
            content_guidelines=[
                "性的表現の制限",
                "暴力表現の適切な警告",
                "著作権遵守"
            ],
            submission_rules=[
                "1日1回まで投稿可能",
                "連載は定期更新推奨"
            ],
            update_frequency_limit="1日1回"
        )

        # カクヨム
        requirements[PublishingPlatform.KAKUYOMU] = PlatformRequirements(
            platform=PublishingPlatform.KAKUYOMU,
            max_title_length=60,
            max_summary_length=600,
            max_content_length=60000,
            supported_formats=[ContentFormat.PLAIN_TEXT, ContentFormat.RICH_TEXT],
            required_metadata=["title", "summary", "genre", "tags"],
            optional_metadata=["content_warning", "target_audience"],
            content_guidelines=[
                "表現の自由重視",
                "適切なタグ付け必須",
                "読者への配慮"
            ],
            submission_rules=[
                "制限なし",
                "品質重視"
            ],
            update_frequency_limit=None
        )

        return requirements

    def _initialize_content_formatters(self) -> dict[ContentFormat, Any]:
        """コンテンツフォーマッターの初期化"""
        return {
            ContentFormat.PLAIN_TEXT: self._format_as_plain_text,
            ContentFormat.HTML: self._format_as_html,
            ContentFormat.MARKDOWN: self._format_as_markdown
        }

    def _initialize_validation_rules(self) -> dict[str, Any]:
        """検証ルールの初期化"""
        return {
            "content_guidelines": {
                "性的表現の制限": self._check_adult_content,
                "暴力表現の適切な警告": self._check_violence_content,
                "著作権遵守": self._check_copyright_compliance
            }
        }

    # ヘルパーメソッドの実装

    def _generate_summary(self, content: str, context: dict[str, Any]) -> str:
        """サマリーの生成"""
        # プロットデータからサマリーを生成
        plot_data = context.get("plot_data", {})
        if plot_data and "summary" in plot_data:
            return plot_data["summary"][:200]  # 200文字以内

        # コンテンツから自動生成
        sentences = content.split("。")
        if len(sentences) >= 3:
            return "。".join(sentences[:3]) + "。"
        return content[:200] + "..." if len(content) > 200 else content

    def _generate_tags(
        self,
        content: str,
        project: ProjectModel,
        context: dict[str, Any]
    ) -> list[str]:
        """タグの生成"""
        tags = []

        # プロジェクトから基本タグ
        if hasattr(project, "genre"):
            tags.append(project.genre)

        # コンテンツから自動タグ生成
        common_words = ["恋愛", "冒険", "ファンタジー", "現代", "学園", "異世界"]
        for word in common_words:
            if word in content:
                tags.append(word)

        return tags[:10]  # 最大10個

    def _estimate_genre(
        self,
        content: str,
        project: ProjectModel,
        context: dict[str, Any]
    ) -> str:
        """ジャンルの推定"""
        # プロジェクトにジャンル設定がある場合
        if hasattr(project, "genre"):
            return project.genre

        # コンテンツから推定
        if any(word in content for word in ["魔法", "魔王", "勇者", "異世界"]):
            return "ファンタジー"
        if any(word in content for word in ["恋", "愛", "好き"]):
            return "恋愛"
        if any(word in content for word in ["学校", "学園", "部活"]):
            return "学園"
        return "その他"

    def _generate_content_warnings(self, content: str) -> list[str]:
        """内容警告の生成"""
        warnings = []

        # 暴力表現のチェック
        violence_words = ["血", "戦い", "殺", "暴力"]
        if any(word in content for word in violence_words):
            warnings.append("暴力表現あり")

        # 性的表現のチェック
        adult_words = ["キス", "恋人", "抱擁"]
        if any(word in content for word in adult_words):
            warnings.append("恋愛表現あり")

        return warnings

    def _adjust_title_length(self, title: str, max_length: int) -> str:
        """タイトル長の調整"""
        if len(title) <= max_length:
            return title
        return title[:max_length-3] + "..."

    def _adjust_content_length(self, content: str, max_length: int) -> str:
        """コンテンツ長の調整"""
        if len(content) <= max_length:
            return content
        # 段落単位で切り詰め
        paragraphs = content.split("\n\n")
        adjusted = ""
        for paragraph in paragraphs:
            if len(adjusted) + len(paragraph) <= max_length - 100:  # 余裕を持つ
                adjusted += paragraph + "\n\n"
            else:
                break
        return adjusted + "[続きは次回...]"

    def _format_as_plain_text(self, content: str) -> str:
        """プレーンテキストフォーマット"""
        return content

    def _format_as_html(self, content: str) -> str:
        """HTMLフォーマット"""
        # 簡易HTML変換
        paragraphs = content.split("\n\n")
        html_content = []

        for paragraph in paragraphs:
            if paragraph.strip():
                html_content.append(f"<p>{paragraph.strip()}</p>")

        return "\n".join(html_content)

    def _format_as_markdown(self, content: str) -> str:
        """Markdownフォーマット"""
        # 簡易Markdown変換
        lines = content.split("\n")
        markdown_content = []

        for line in lines:
            if line.strip():
                markdown_content.append(line)
            else:
                markdown_content.append("")

        return "\n".join(markdown_content)

    def _validate_formatted_content(
        self,
        content: str,
        requirements: PlatformRequirements
    ) -> tuple[bool, list[str]]:
        """フォーマット済みコンテンツの検証"""
        warnings = []

        # 基本的な検証
        if len(content) == 0:
            return False, ["コンテンツが空です"]

        # 文字エンコーディングチェック
        try:
            content.encode("utf-8")
        except UnicodeEncodeError:
            warnings.append("文字エンコーディングに問題があります")

        # 改行コードの統一チェック
        if "\r\n" in content:
            warnings.append("改行コードの統一が推奨されます")

        return True, warnings

    def _generate_formatting_notes(
        self,
        platform: PublishingPlatform,
        format: ContentFormat
    ) -> list[str]:
        """フォーマット注意事項の生成"""
        notes = []

        if platform == PublishingPlatform.NAROU:
            notes.append("小説家になろう形式でフォーマット済み")
            if format == ContentFormat.PLAIN_TEXT:
                notes.append("プレーンテキスト形式を使用")

        elif platform == PublishingPlatform.KAKUYOMU:
            notes.append("カクヨム形式でフォーマット済み")
            if format == ContentFormat.HTML:
                notes.append("HTMLフォーマットを適用")

        return notes

    def _check_content_guidelines(
        self,
        content: FormattedContent,
        guidelines: list[str]
    ) -> dict[str, list[str]]:
        """コンテンツガイドライン準拠チェック"""

        result = {
            "passed": [],
            "failed": [],
            "warnings": [],
            "blocking": []
        }

        for guideline in guidelines:
            if guideline == "性的表現の制限":
                if self._check_adult_content(content.content):
                    result["warnings"].append("性的表現が含まれています")
                else:
                    result["passed"].append("性的表現チェック通過")

            elif guideline == "暴力表現の適切な警告":
                if self._check_violence_content(content.content):
                    if "暴力表現あり" in content.metadata.content_warning:
                        result["passed"].append("暴力表現警告適切")
                    else:
                        result["warnings"].append("暴力表現の警告が推奨されます")
                else:
                    result["passed"].append("暴力表現なし")

            elif guideline == "著作権遵守":
                if self._check_copyright_compliance(content.content):
                    result["passed"].append("著作権問題なし")
                else:
                    result["failed"].append("著作権問題の可能性")
                    result["blocking"].append("著作権問題要確認")

        return result

    def _check_adult_content(self, content: str) -> bool:
        """成人向けコンテンツのチェック"""
        adult_indicators = ["性的", "アダルト", "18禁"]
        return any(indicator in content for indicator in adult_indicators)

    def _check_violence_content(self, content: str) -> bool:
        """暴力表現のチェック"""
        violence_indicators = ["血", "殺", "暴力", "戦闘"]
        return any(indicator in content for indicator in violence_indicators)

    def _check_copyright_compliance(self, content: str) -> bool:
        """著作権遵守のチェック"""
        # 簡易実装：特定のキーワードをチェック
        copyright_violations = ["コピー", "転載", "無断使用"]
        return not any(violation in content for violation in copyright_violations)

    def _generate_preparation_summary(
        self,
        formatted_contents: list[FormattedContent],
        validations: list[PublishingValidation],
        readiness_score: float
    ) -> str:
        """準備サマリーの生成"""

        total_platforms = len(formatted_contents)
        ready_platforms = len([v for v in validations if v.ready_for_publishing])

        summary_parts = [
            f"対象プラットフォーム: {total_platforms}個",
            f"公開準備完了: {ready_platforms}個",
            f"準備完了率: {readiness_score:.1%}"
        ]

        if readiness_score >= 0.9:
            summary_parts.append("即座に公開可能です")
        elif readiness_score >= 0.7:
            summary_parts.append("軽微な調整後に公開可能です")
        elif readiness_score >= 0.5:
            summary_parts.append("追加作業が必要です")
        else:
            summary_parts.append("大幅な修正が必要です")

        return " | ".join(summary_parts)

    def _generate_next_actions(
        self,
        validations: list[PublishingValidation],
        readiness_score: float
    ) -> list[str]:
        """次のアクションの生成"""

        actions = []

        if readiness_score >= 0.9:
            actions.append("✅ 公開実行準備完了")
            actions.append("📅 公開スケジュールの確認")
            actions.append("🔄 最終確認チェックリスト実行")

        elif readiness_score >= 0.7:
            actions.append("🔧 軽微な修正の実行")
            for validation in validations:
                if not validation.ready_for_publishing and validation.warnings:
                    actions.append(f"⚠️ {validation.platform.value}: {validation.warnings[0]}")
            actions.append("🔄 修正後の再検証")

        else:
            actions.append("🚨 重要な問題の解決が必要")
            for validation in validations:
                if validation.blocking_issues:
                    for issue in validation.blocking_issues[:2]:  # 最大2個まで表示
                        actions.append(f"❗ {validation.platform.value}: {issue}")
            actions.append("📋 品質改善計画の策定")

        return actions



# application/use_cases/narrative_depth_evaluation.py
"""内面描写深度評価のユースケース"""

from typing import Any

from noveler.domain.repositories.analysis_result_repository import AnalysisResultRepository
from noveler.domain.repositories.manuscript_repository import ManuscriptRepository
from noveler.domain.repositories.plot_repository import PlotRepository
from noveler.domain.services.narrative_depth_analyzer import NarrativeDepthAnalyzer
from noveler.domain.services.viewpoint_evaluator import ViewpointEvaluator
from noveler.domain.value_objects.project_time import ProjectTimezone, project_now

# JSTタイムゾーン
JST = ProjectTimezone.jst().timezone


class EvaluateNarrativeDepthUseCase:
    """内面描写深度評価ユースケース"""

    def __init__(
        self,
        plot_repository: PlotRepository,
        text_repository: ManuscriptRepository,
        result_repository: AnalysisResultRepository,
        analyzer: NarrativeDepthAnalyzer,
        evaluator: ViewpointEvaluator,
    ) -> None:
        self.plot_repository = plot_repository
        self.text_repository = text_repository
        self.result_repository = result_repository
        self.analyzer = analyzer
        self.evaluator = evaluator

    def execute(self, episode_number: int) -> dict[str, Any]:
        """内面描写深度評価を実行"""
        # 1. エピソードテキストを取得
        episode_text = self.text_repository.get_episode_text(episode_number)
        if not episode_text:
            return self._create_error_result(f"Episode {episode_number} not found")

        # 2. 視点情報を取得
        viewpoint_info = self.plot_repository.get_viewpoint_info(episode_number)
        if not viewpoint_info:
            return self._create_error_result(f"Viewpoint info for episode {episode_number} not found")

        # 3. 内面描写深度分析を実行
        depth_score = self.analyzer.analyze_depth(episode_text)

        # 4. 視点情報に基づいて調整
        viewpoint_type = viewpoint_info.get("視点タイプ", "標準")
        complexity_level = viewpoint_info.get("複雑度", "中")

        adjusted_score = self.evaluator.adjust_for_viewpoint(
            depth_score,
            viewpoint_type,
            complexity_level,
        )

        # 5. 結果を構築
        result = self._create_success_result(
            episode_number,
            depth_score,
            adjusted_score,
            viewpoint_info,
        )

        # 6. 結果を保存
        self.result_repository.save_evaluation_result(episode_number, result)

        return result

    def _create_success_result(
        self, episode_number: int, depth_score: float, adjusted_score: float, viewpoint_info: dict[str, Any]
    ) -> dict[str, Any]:
        """成功結果を作成"""
        return {
            "status": "success",
            "episode_number": episode_number,
            "narrative_depth_analysis": {
                "layer_scores": {
                    layer.value: {
                        "score": score.score,
                        "percentage": score.percentage,
                        "max_score": score.max_score,
                    }
                    for layer, score in depth_score.layer_scores.items()
                },
                "base_total_score": depth_score.total_score,
                "adjusted_total_score": adjusted_score,
                "organic_combination_bonus": depth_score._has_organic_combination(),
            },
            "viewpoint_info": viewpoint_info,
            "recommendations": self._generate_recommendations(depth_score, viewpoint_info),
            "evaluation_metadata": {
                "analyzer_version": "1.0.0",
                "evaluation_timestamp": self._get_current_timestamp(),
            },
        }

    def _create_error_result(self, error_message: str) -> dict[str, Any]:
        """エラー結果を作成"""
        return {
            "status": "error",
            "error_message": error_message,
            "evaluation_metadata": {
                "analyzer_version": "1.0.0",
                "evaluation_timestamp": self._get_current_timestamp(),
            },
        }

    def _generate_recommendations(self, depth_score: float, viewpoint_info: dict[str, Any]) -> list[str]:
        """推奨事項を生成"""
        recommendations = []

        # 各層のスコアに基づく推奨事項
        for layer, score in depth_score.layer_scores.items():
            if score.percentage < 30:
                recommendations.append(
                    {
                        "type": "depth_improvement",
                        "layer": layer,
                        "current_score": score.percentage,
                        "message": f"{layer}の描写を深めることをお勧めします",
                    }
                )

        # 総合スコアに基づく推奨事項
        if depth_score.total_score < 50:
            recommendations.append(
                {"type": "overall_improvement", "message": "全体的な内面描写を充実させることをお勧めします"}
            )

        # 視点タイプに基づく推奨事項
        viewpoint_type = viewpoint_info.get("視点タイプ", "")
        if "単一視点" in viewpoint_type and depth_score.total_score < 60:
            recommendations.append({"type": "viewpoint_specific", "message": "単一視点ではより深い内面描写が有効です"})

        return recommendations

    def _get_current_timestamp(self) -> str:
        """現在のタイムスタンプを取得"""

        return project_now().datetime.isoformat()


class ViewpointAwareQualityEvaluationUseCase:
    """視点情報を考慮した品質評価ユースケース"""

    def __init__(self, narrative_evaluation: EvaluateNarrativeDepthUseCase, plot_repository: PlotRepository) -> None:
        self.narrative_evaluation = narrative_evaluation
        self.plot_repository = plot_repository

    def execute(self, episode_number: int, base_scores: dict[str, float]) -> dict[str, Any]:
        """視点情報を考慮した総合品質評価を実行"""
        # 1. 視点情報を取得
        viewpoint_info = self.plot_repository.get_viewpoint_info(episode_number)
        if not viewpoint_info:
            return self._apply_standard_evaluation(base_scores)

        # 2. 内面描写深度評価を実行
        narrative_result = self.narrative_evaluation.execute(episode_number)
        if narrative_result["status"] != "success":
            return self._apply_standard_evaluation(base_scores)

        # 3. 視点タイプに基づいてスコアを調整
        adjusted_scores = self._adjust_scores_by_viewpoint(
            base_scores,
            narrative_result,
            viewpoint_info,
        )

        # 4. 結果を構築
        return {
            "status": "success",
            "episode_number": episode_number,
            "viewpoint_adjusted_scores": adjusted_scores,
            "base_scores": base_scores,
            "narrative_depth_analysis": narrative_result.get("narrative_depth_analysis"),
            "viewpoint_info": viewpoint_info,
            "adjustment_summary": self._create_adjustment_summary(
                base_scores,
                adjusted_scores,
                viewpoint_info,
            ),
        }

    def _adjust_scores_by_viewpoint(
        self, base_scores: dict[str, float], narrative_result: dict[str, Any], viewpoint_info: dict[str, Any]
    ) -> dict[str, float]:
        """視点タイプに基づいてスコアを調整"""
        adjusted_scores = base_scores.copy()
        viewpoint_type = viewpoint_info.get("視点タイプ", "")

        # 内面描写深度スコアを取得
        narrative_score = narrative_result["narrative_depth_analysis"]["adjusted_total_score"]

        if "内省型" in viewpoint_type or "単一視点" in viewpoint_type:
            # 単一視点・内省型:会話比率の重みを下げ、内面描写を重視
            if "dialogue_ratio" in adjusted_scores:
                adjusted_scores["dialogue_ratio"] *= 0.5
            adjusted_scores["narrative_depth"] = narrative_score * 1.5

        elif "身体交換" in viewpoint_type:
            # 身体交換:内面描写を重視
            adjusted_scores["narrative_depth"] = narrative_score * 1.3

        elif "複数視点" in viewpoint_type:
            # 複数視点:視点の明確さを重視
            if "viewpoint_clarity" in adjusted_scores:
                adjusted_scores["viewpoint_clarity"] *= 1.5

        else:
            # 標準的な評価
            adjusted_scores["narrative_depth"] = narrative_score

        return adjusted_scores

    def _create_adjustment_summary(
        self, base_scores: dict[str, float], adjusted_scores: dict[str, float], viewpoint_info: dict[str, Any]
    ) -> dict[str, Any]:
        """調整サマリーを作成"""
        adjustments = {}
        for key, base_val in base_scores.items():
            if key in adjusted_scores:
                adjusted_val = adjusted_scores[key]
                if abs(base_val - adjusted_val) > 0.1:
                    adjustments[key] = {
                        "base": base_val,
                        "adjusted": adjusted_val,
                        "change": adjusted_val - base_val,
                    }

        return {
            "viewpoint_type": viewpoint_info.get("視点タイプ", "不明"),
            "complexity_level": viewpoint_info.get("複雑度", "不明"),
            "score_adjustments": adjustments,
            "adjustment_reason": self._get_adjustment_reason(viewpoint_info),
        }

    def _get_adjustment_reason(self, viewpoint_info: dict[str, Any]) -> str:
        """調整理由を取得"""
        viewpoint_type = viewpoint_info.get("視点タイプ", "")

        if "内省型" in viewpoint_type:
            return "内省型エピソードのため会話比率は参考値、内面描写を重視して評価"
        if "身体交換" in viewpoint_type:
            return "身体交換エピソードのため内面描写を重視して評価"
        if "複数視点" in viewpoint_type:
            return "複数視点エピソードのため視点の明確さを重視して評価"
        return "標準的な評価基準を適用"

    def _apply_standard_evaluation(self, base_scores: dict[str, float]) -> dict[str, Any]:
        """標準評価を適用"""
        return {
            "status": "standard_evaluation",
            "adjusted_scores": base_scores,
            "base_scores": base_scores,
            "adjustment_summary": {
                "viewpoint_type": "不明",
                "adjustment_reason": "視点情報が取得できないため標準評価を適用",
            },
        }



#!/usr/bin/env python3
"""循環インポート解消テスト

SPEC-CIRC-001: 循環インポート解消とアーキテクチャ修正
"""

import importlib
import sys

import pytest


class TestCircularImportResolution:
    """循環インポート解消テスト"""

    @pytest.mark.spec("SPEC-CIRC-001")
    def test_no_circular_import_container(self):
        """container.pyが循環インポートなしでインポート可能"""
        if "noveler.infrastructure.di.container" in sys.modules:
            del sys.modules["noveler.infrastructure.di.container"]

        from noveler.infrastructure.di.container import auto_setup_container

        assert callable(auto_setup_container)

    @pytest.mark.spec("SPEC-CIRC-001")
    def test_environment_setup_independent_import(self):
        """environment_setup.pyが独立してインポート可能"""
        module_name = "noveler.presentation.cli.environment_setup"
        if module_name in sys.modules:
            del sys.modules[module_name]

        try:
            from noveler.presentation.cli.environment_setup import initialize_environment
        except ModuleNotFoundError:
            pytest.skip("environment_setup モジュールが存在しないためスキップ")
        else:
            assert callable(initialize_environment)

    @pytest.mark.spec("SPEC-CIRC-001")
    def test_mcp_server_startup_capability(self):
        """MCPサーバー起動に必要なモジュールがインポート可能"""
        # 現在は失敗するはず（RED状態）
        try:
            from noveler.presentation.cli.novel_cli import main
        except ModuleNotFoundError:
            pytest.skip("novel_cli モジュールが存在しないためスキップ")
        else:
            from noveler.infrastructure.di.container import auto_setup_container
            assert callable(main)
            assert callable(auto_setup_container)

    @pytest.mark.spec("SPEC-CIRC-001")
    def test_shared_utilities_independence(self):
        """shared_utilitiesがinfrastructure層から独立している"""
        # DDD違反の検証（現在は違反状態でRED）

        # 循環の原因となっているファイルを確認
        problematic_imports = [
            "noveler.infrastructure.yaml_project_settings_repository",
            "noveler.infrastructure.adapters.prompt_generation_adapter",
            "noveler.infrastructure.config.project_detector"
        ]

        for module_name in problematic_imports:
            if module_name in sys.modules:
                del sys.modules[module_name]

            module = importlib.import_module(module_name)
            assert module is not None


class TestArchitectureCompliance:
    """アーキテクチャ準拠テスト"""

    @pytest.mark.skip(reason="Infrastructure layer DDD violations - known technical debt to be resolved in Phase 4")
    @pytest.mark.spec("SPEC-CIRC-001")
    def test_infrastructure_layer_isolation(self):
        """インフラ層がプレゼンテーション層に依存していない"""
        # 現在はDDD違反状態（RED）
        # 既知の技術負債: adaptersフォルダと特定ファイルが presentation 層に依存

        # インフラ層のファイルがプレゼンテーション層をインポートしていないかチェック
        from pathlib import Path

        infrastructure_files = list(Path("src/noveler/infrastructure").rglob("*.py"))
        violations = []
        known_violations = {
            "src\\noveler\\infrastructure\\claude_code_environment_manager.py",
            "src\\noveler\\infrastructure\\claude_code_session_integration.py",
            "src\\noveler\\infrastructure\\simple_quality_record_creator.py",
        }

        # adapters フォルダのファイルは既知の技術負債として除外
        adapter_files = {"src\\noveler\\infrastructure\\adapters"}

        for file_path in infrastructure_files[:80]:  # より広範なサンプル検査
            try:
                with open(file_path, encoding="utf-8") as f:
                    content = f.read()

                file_path_str = str(file_path)

                # adapters フォルダはスキップ（既知の技術負債）
                if any(adapter in file_path_str for adapter in adapter_files):
                    continue

                # プレゼンテーション層への依存をチェック
                if "noveler.presentation" in content and file_path_str not in known_violations:
                    violations.append(file_path_str)

            except Exception:
                continue

        # 既知の違反については許容（技術負債として追跡）
        assert not violations, f"新しいプレゼンテーション層への依存が発見されました: {violations}"



# File: src/noveler/application/services/stepwise_execution_service.py
# Purpose: Re-export base class and response wrapper from Domain layer
#          for backward-compatible application layer integration.
# Context: Acts as a bridge so legacy service imports continue to work
#          after moving BaseWritingStep to Domain layer.

from noveler.domain.interfaces.writing_step_protocol import BaseWritingStep
from noveler.domain.interfaces.writing_step_protocol import WritingStepResponse

import inspect
import time
from dataclasses import dataclass
from typing import Any


@dataclass
class WritingStepResponse:
    """Generic response container for stepwise services.

    Provides a tolerant constructor so legacy services can attach arbitrary
    metadata without raising errors (mirrors behaviour from the retired shim).
    """

    success: bool
    step_number: int
    step_name: str
    execution_time_ms: float = 0.0
    error_message: str | None = None
    data: dict[str, Any] | None = None
    metadata: dict[str, Any] | None = None

    def __init__(
        self,
        success: bool,
        step_number: int,
        step_name: str,
        execution_time_ms: float = 0.0,
        error_message: str | None = None,
        data: dict[str, Any] | None = None,
        metadata: dict[str, Any] | None = None,
        **extra: Any,
    ) -> None:
        self.success = success
        self.step_number = step_number
        self.step_name = step_name
        self.execution_time_ms = execution_time_ms
        self.error_message = error_message
        self.data = data
        self.metadata = metadata or {}
        # Preserve unknown attributes for downstream compatibility.
        for key, value in extra.items():
            setattr(self, key, value)


class BaseWritingStep:
    """Minimal compatibility layer for legacy stepwise services.

    Legacy services expect to override `execute_step`, `get_step_name`, and
    `get_step_description`. The new shim wraps those methods and exposes the
    asynchronous `execute` coroutine needed by the updated use case.
    """

    STEP_NUMBER: int = -1
    STEP_NAME: str = "step"

    def __init__(
        self,
        step_number: int | None = None,
        step_name: str | None = None,
        **_: Any,
    ) -> None:
        self.step_number = step_number if step_number is not None else getattr(self, "STEP_NUMBER", -1)
        self.step_name = step_name if step_name is not None else getattr(self, "STEP_NAME", self.__class__.__name__)

    async def execute(
        self,
        episode_number: int,
        previous_results: dict[int, Any] | None = None,
    ) -> WritingStepResponse:
        """Async wrapper that delegates to the synchronous legacy implementation."""
        start_time = time.time()
        context = self._prepare_context(episode_number, previous_results or {})

        try:
            result = self.execute_step(context)
            if inspect.isawaitable(result):
                result = await result

            execution_time = (time.time() - start_time) * 1000

            if isinstance(result, WritingStepResponse):
                # Ensure timing metadata is populated for downstream consumers.
                if not getattr(result, "execution_time_ms", 0.0):
                    result.execution_time_ms = execution_time
                return result

            payload: dict[str, Any]
            if isinstance(result, dict):
                payload = result
            else:
                payload = {"result": result}

            return WritingStepResponse(
                success=True,
                step_number=self.step_number,
                step_name=self.step_name,
                execution_time_ms=execution_time,
                data=payload,
            )

        except Exception as exc:  # pragma: no cover - defensive compatibility path
            execution_time = (time.time() - start_time) * 1000
            return WritingStepResponse(
                success=False,
                step_number=self.step_number,
                step_name=self.step_name,
                execution_time_ms=execution_time,
                error_message=str(exc),
            )

    def get_step_name(self) -> str:
        """Return the human-readable step name."""
        return getattr(self, "STEP_NAME", self.step_name)

    def get_step_description(self) -> str:
        """Return the step description if provided by subclasses."""
        return getattr(self, "STEP_DESCRIPTION", "")

    def execute_step(self, context: dict[str, Any]) -> Any:  # pragma: no cover - abstract shim
        """Legacy synchronous hook implemented by subclasses."""
        raise NotImplementedError("execute_step must be implemented by subclasses.")

    def _prepare_context(
        self,
        episode_number: int,
        previous_results: dict[int, Any],
    ) -> dict[str, Any]:
        """Assemble the context dict expected by legacy services."""
        context: dict[str, Any] = {
            "episode_number": episode_number,
            "previous_results": previous_results,
        }

        # Attempt to recover commonly used data (e.g., project info) from prior steps.
        project = None
        for result in previous_results.values():
            if hasattr(result, "project_root"):
                project = result.project_root
                break
            if hasattr(result, "data") and isinstance(result.data, dict) and "project" in result.data:
                project = result.data["project"]
                break

        if project:
            context["project"] = project

        return context



# File: src/noveler/domain/aggregates/infrastructure_service_catalog.py
# Purpose: Manage infrastructure service definitions, dependencies, and invariants.
# Context: Replaces responsibilities previously embedded in InfrastructureIntegrationAggregate.

"""Infrastructure service catalog aggregate."""

from __future__ import annotations

from collections.abc import Iterable
from dataclasses import dataclass, field

from noveler.domain.exceptions import DomainException
from noveler.domain.value_objects.infrastructure_service_definition import ServiceDefinition


@dataclass
class _DependencyGraph:
    """Internal helper for dependency tracking."""

    edges: dict[str, set[str]] = field(default_factory=dict)

    def add(self, service: str, dependency: str) -> None:
        """Record dependency."""
        self.edges.setdefault(service, set()).add(dependency)

    def remove_service(self, service: str, *, preserve_inbound: bool = False) -> None:
        """Remove service from graph."""
        self.edges.pop(service, None)
        if not preserve_inbound:
            for deps in self.edges.values():
                deps.discard(service)

    def has_cycle(self) -> bool:
        """
        Detect cyclic dependencies in the service dependency graph.

        Purpose:
            Identify circular dependencies using depth-first search (DFS),
            preventing invalid dependency chains in service definitions.

        Returns:
            True if any cycle exists in the dependency graph.

        Side Effects:
            None (read-only cycle detection).
        """
        visited: set[str] = set()
        stack: set[str] = set()

        def visit(node: str) -> bool:
            """
            Visit node and detect cycles via DFS stack tracking.

            Purpose:
                Recursively explore dependencies, tracking the current path
                to detect back-edges indicating cycles.

            Args:
                node: Current node (service name) being visited.

            Returns:
                True if cycle detected from this node.
            """
            if node in stack:
                return True
            if node in visited:
                return False
            visited.add(node)
            stack.add(node)
            for neighbour in self.edges.get(node, ()):  # pragma: no branch - simple iteration
                if visit(neighbour):
                    return True
            stack.remove(node)
            return False

        return any(visit(node) for node in list(self.edges))

    def toposort(self) -> list[str]:
        """Return nodes in dependency order."""
        indegree: dict[str, int] = {}
        adjacency: dict[str, set[str]] = {}
        nodes = set(self.edges)
        for deps in self.edges.values():
            nodes.update(deps)
        for node in nodes:
            indegree[node] = 0
        for service, deps in self.edges.items():
            indegree.setdefault(service, 0)
            indegree[service] += len(deps)
            for dep in deps:
                adjacency.setdefault(dep, set()).add(service)
        queue = [node for node, degree in indegree.items() if degree == 0]
        ordered: list[str] = []
        while queue:
            node = queue.pop(0)
            ordered.append(node)
            for dependent in adjacency.get(node, ()):
                indegree[dependent] -= 1
                if indegree[dependent] == 0:
                    queue.append(dependent)
        if len(ordered) != len(indegree):
            msg = "Cycle detected while ordering services"
            raise DomainException(msg)
        return ordered


class InfrastructureServiceCatalog:
    """Aggregate storing infrastructure service definitions."""

    def __init__(self, project_id: str) -> None:
        """
        Initialize the infrastructure service catalog aggregate.

        Purpose:
            Create a service catalog bound to a project and initialize
            empty service registry and dependency graph structures.

        Args:
            project_id: Unique identifier of the owning project.

        Side Effects:
            Initialize instance attributes (service registry, dependency graph).
        """
        self._project_id = project_id
        self._services: dict[str, ServiceDefinition] = {}
        self._dependency_graph = _DependencyGraph()

    @property
    def project_id(self) -> str:
        """Return project identifier."""
        return self._project_id

    def register(self, definition: ServiceDefinition, *, override: bool = False) -> None:
        """Register service definition."""
        if definition.name in self._services and not override:
            msg = f"Service '{definition.name}' is already registered"
            raise DomainException(msg)

        self._validate_dependencies(definition)
        if override:
            self._dependency_graph.remove_service(definition.name, preserve_inbound=True)

        self._services[definition.name] = definition
        for dependency in definition.dependencies:
            self._dependency_graph.add(definition.name, dependency)

        if self._dependency_graph.has_cycle():
            self._dependency_graph.remove_service(definition.name)
            self._services.pop(definition.name, None)
            msg = f"Cyclic dependency detected when adding '{definition.name}'"
            raise DomainException(msg)

    def get(self, service_name: str) -> ServiceDefinition | None:
        """Return definition by name."""
        return self._services.get(service_name)

    def list_all(self) -> Iterable[ServiceDefinition]:
        """Iterate over registered services."""
        return list(self._services.values())

    def ordered_services(self) -> list[ServiceDefinition]:
        """Return services respecting dependency order."""
        ordered_names = self._dependency_graph.toposort()
        return [self._services[name] for name in ordered_names if name in self._services]

    def _validate_dependencies(self, definition: ServiceDefinition) -> None:
        """Ensure dependencies exist or will be registered later."""
        unresolved = [
            dependency
            for dependency in definition.dependencies
            if dependency not in self._services and dependency != definition.name
        ]
        if unresolved:
            missing = ", ".join(sorted(unresolved))
            msg = f"Dependencies missing for '{definition.name}': {missing}"
            raise DomainException(msg)



"""原稿生成サービスファクトリー

ゴールデンサンプルのbootstrapパターンに基づいた依存性注入実装。
"""


from noveler.domain.interfaces.logger_service_protocol import ILoggerService
from noveler.domain.interfaces.path_service_protocol import IPathService
from noveler.domain.services.writing_steps.manuscript_generator_service import (
    ManuscriptGeneratorService,
)
from noveler.infrastructure.factories.configuration_service_factory import (
    ConfigurationManager,
    get_configuration_manager,
)
from noveler.infrastructure.factories.logger_service_factory import get_logger_service
from noveler.infrastructure.factories.path_service_factory import get_path_service


def create_manuscript_generator(
    logger_service: ILoggerService | None = None,
    path_service: IPathService | None = None,
    config_manager: ConfigurationManager | None = None,
    **overrides
) -> ManuscriptGeneratorService:
    """原稿生成サービスのファクトリー関数

    ゴールデンサンプルのbootstrapパターンに基づき、
    依存性を解決してサービスインスタンスを生成する。

    Args:
        logger_service: ロガーサービス（Noneの場合はデフォルトを使用）
        path_service: パスサービス（Noneの場合はデフォルトを使用）
        config_manager: 設定管理サービス（Noneの場合はデフォルトを使用）
        **overrides: その他のオーバーライド設定

    Returns:
        設定済みのManuscriptGeneratorServiceインスタンス

    Example:
        # デフォルト設定で生成
        service = create_manuscript_generator()

        # カスタム依存性で生成
        custom_logger = MyCustomLogger()
        service = create_manuscript_generator(logger_service=custom_logger)

        # テスト用のモック注入
        mock_config = MockConfigurationManager()
        service = create_manuscript_generator(config_manager=mock_config)
    """
    # デフォルト依存性の解決
    if logger_service is None:
        logger_service = get_logger_service("manuscript_generator")

    if path_service is None:
        path_service = get_path_service()

    if config_manager is None:
        config_manager = get_configuration_manager()

    # サービスインスタンスの生成
    return ManuscriptGeneratorService(
        logger_service=logger_service,
        path_service=path_service,
        config_manager=config_manager,
        **overrides
    )


def create_test_manuscript_generator(**overrides) -> ManuscriptGeneratorService:
    """テスト用の原稿生成サービスを作成

    テスト環境用の設定で原稿生成サービスを生成する。

    Args:
        **overrides: オーバーライド設定

    Returns:
        テスト用に設定されたManuscriptGeneratorServiceインスタンス

    Example:
        # テスト用サービスの生成
        service = create_test_manuscript_generator()

        # カスタム設定でテスト
        service = create_test_manuscript_generator(
            target_word_count=1000,
            dry_run=True
        )
    """
    # テスト用のモックまたは軽量実装を使用
    # 実際のファイルI/OやAPI呼び出しを避ける

    # 必要に応じてテスト用の依存性を設定
    test_logger = get_logger_service("test_manuscript_generator")
    test_path = get_path_service()

    # テスト用の設定マネージャー（キャッシュやファイルアクセスを最小限に）
    test_config = get_configuration_manager()

    return create_manuscript_generator(
        logger_service=test_logger,
        path_service=test_path,
        config_manager=test_config,
        **overrides
    )



#!/usr/bin/env python3
"""CLI レスポンス→JSON変換器"""

import json
from pathlib import Path
from typing import Any

from noveler.domain.value_objects.project_time import project_now
from noveler.infrastructure.json.converters.base_converter import BaseConverter
from noveler.infrastructure.json.file_managers.file_reference_manager import FileReferenceManager
from noveler.infrastructure.json.models.file_reference_models import FileReferenceCollection
from noveler.infrastructure.json.models.response_models import (
    ErrorDetailModel,
    ErrorResponseModel,
    StandardResponseModel,
)


class CLIResponseConverter(BaseConverter):
    """CLI レスポンス→JSON変換器"""

    def __init__(
        self,
        schema_dir: Path | None = None,
        output_dir: Path | None = None,
        validate_schema: bool = True
    ) -> None:
        super().__init__(schema_dir, output_dir, validate_schema)
        self.file_manager = FileReferenceManager(self.output_dir)

    def convert(self, cli_result: dict[str, Any]) -> dict[str, Any]:
        """CLI実行結果をJSON形式に変換"""

        start_time = project_now().datetime

        try:
            if cli_result.get("success", False):
                result = self._convert_success_response(cli_result)
            else:
                result = self._convert_error_response(cli_result)

            # 実行時間追加（存在しない場合のみ）
            if "execution_time_ms" not in result:
                end_time = project_now().datetime
                result["execution_time_ms"] = (end_time - start_time).total_seconds() * 1000

            return result

        except Exception as e:
            return self._create_emergency_error_response(str(e), cli_result)

    def _convert_success_response(self, cli_result: dict[str, Any]) -> dict[str, Any]:
        """成功レスポンス変換"""

        file_references = []

        # コンテンツをファイル参照に変換
        if "content" in cli_result:
            file_ref = self.file_manager.save_content(
                content=cli_result["content"],
                content_type="text/markdown",
                filename_prefix=cli_result.get("command", "output").replace(" ", "_"),
            )
            file_references.append(file_ref)

        if "yaml_content" in cli_result:
            file_ref = self.file_manager.save_content(
                content=cli_result["yaml_content"],
                content_type="text/yaml",
                filename_prefix=f"{cli_result.get('command', 'output').replace(' ', '_')}_config",
            )
            file_references.append(file_ref)

        if "json_data" in cli_result:
            file_ref = self.file_manager.save_content(
                content=json.dumps(cli_result["json_data"], ensure_ascii=False, indent=2),
                content_type="application/json",
                filename_prefix=f"{cli_result.get('command', 'output').replace(' ', '_')}_data",
            )
            file_references.append(file_ref)

        # FileReferenceCollectionモデル作成
        file_collection = FileReferenceCollection(
            files=file_references,
            total_files=len(file_references),
            total_size_bytes=sum(f.size_bytes for f in file_references),
        )

        # StandardResponseModel作成・バリデーション
        response_data = {
            "success": True,
            "command": cli_result.get("command", "unknown"),
            "execution_time_ms": cli_result.get("execution_time_ms", 0.0),
            "outputs": file_collection.model_dump(),
            "metadata": self._extract_metadata(cli_result),
            "created_at": project_now().datetime,
        }

        response_model = StandardResponseModel(**response_data)
        return response_model.model_dump()

    def _convert_error_response(self, cli_result: dict[str, Any]) -> dict[str, Any]:
        """エラーレスポンス変換"""

        error_detail = ErrorDetailModel(
            code=cli_result.get("error_code", "UNKNOWN_ERROR"),
            message=cli_result.get("error_message", "不明なエラーが発生しました"),
            hint=cli_result.get("error_hint", "ログを確認し、必要に応じてサポートに連絡してください"),
            details=cli_result.get("error_details", {}),
            stack_trace=cli_result.get("stack_trace") if cli_result.get("debug_mode") else None,
        )

        error_data = {
            "success": False,
            "error": error_detail.model_dump(),
            "command": cli_result.get("command", "unknown"),
            "created_at": project_now().datetime,
        }

        # ErrorResponseModel作成・バリデーション
        error_model = ErrorResponseModel(**error_data)
        return error_model.model_dump()

    def _extract_metadata(self, cli_result: dict[str, Any]) -> dict[str, Any]:
        """メタデータ抽出"""
        metadata_keys = [
            "session_id",
            "user_id",
            "project_id",
            "environment",
            "version",
            "git_commit",
            "performance_metrics",
            "word_count",
            "character_count",
            "quality_score",
        ]

        metadata = {}

        # トップレベルからメタデータキーを抽出
        for key in metadata_keys:
            if key in cli_result:
                metadata[key] = cli_result[key]

        # 'metadata'キー内からも抽出
        if "metadata" in cli_result and isinstance(cli_result["metadata"], dict):
            for key in metadata_keys:
                if key in cli_result["metadata"]:
                    metadata[key] = cli_result["metadata"][key]

        return metadata

    def _create_emergency_error_response(self, error_msg: str, original_data: dict) -> dict[str, Any]:
        """緊急エラーレスポンス生成"""
        return {
            "success": False,
            "error": {
                "code": "CONVERTER_ERROR",
                "message": f"JSON変換中にエラーが発生しました: {error_msg}",
                "hint": "開発者に連絡してください。原因調査のため元データを保存しています。",
                "details": {"original_data_keys": list(original_data.keys()), "conversion_error": error_msg},
            },
            "command": original_data.get("command", "unknown"),
            "created_at": project_now().datetime.isoformat(),
        }



"""
# File: tests/contracts/test_template_repository_contract.py
# Purpose: Contract test (baseline) for FileTemplateRepository search order and UTF-8 loading.
# Context: Phase C minimal — keeps behavior stable across refactors.

This test verifies the observable contract of noveler.infrastructure.repositories.file_template_repository:
- find_template(step_id:int, step_slug:str) searches in the documented order and returns the first match.
- load_template_content reads UTF-8 text and raises on missing files.

The test builds a temporary directory structure; it does not depend on project templates.
"""

from __future__ import annotations

from pathlib import Path

import pytest

from noveler.infrastructure.repositories.file_template_repository import FileTemplateRepository


def _scaffold(base: Path) -> None:
    (base / "quality" / "checks").mkdir(parents=True, exist_ok=True)
    (base / "quality" / "checks" / "backup").mkdir(parents=True, exist_ok=True)
    (base / "writing").mkdir(parents=True, exist_ok=True)


def test_find_template_search_order_first_match_wins(tmp_path: Path) -> None:
    base = tmp_path / "templates"
    _scaffold(base)

    # Prepare filenames
    step_id = 1
    slug = "typo_check"
    filename = f"check_step{step_id:02d}_{slug}.yaml"

    # Case 1: only in writing -> returns writing
    writing = base / "writing" / filename
    writing.write_text("version: 1\n", encoding="utf-8")

    repo = FileTemplateRepository(base)
    found = repo.find_template(step_id, slug)
    assert found == writing

    # Case 2: adding backup should take precedence over writing
    backup = base / "quality" / "checks" / "backup" / filename
    backup.write_text("version: 1\n", encoding="utf-8")
    found2 = repo.find_template(step_id, slug)
    assert found2 == backup

    # Case 3: adding checks should take highest precedence
    checks = base / "quality" / "checks" / filename
    checks.write_text("version: 1\n", encoding="utf-8")
    found3 = repo.find_template(step_id, slug)
    assert found3 == checks


def test_load_template_content_utf8_and_missing(tmp_path: Path) -> None:
    base = tmp_path / "templates"
    _scaffold(base)
    p = base / "writing" / "check_step01_sample.yaml"
    text = "日本語UTF-8テンプレート\nkey: 値\n"
    p.write_text(text, encoding="utf-8")

    repo = FileTemplateRepository(base)
    content = repo.load_template_content(p)
    assert "key: 値" in content

    with pytest.raises(FileNotFoundError):
        repo.load_template_content(p.with_name("missing.yaml"))




#!/usr/bin/env python3
# File: tests/unit/infrastructure/logging/test_safe_rotating_file_handler.py
# Purpose: Validate SafeRotatingFileHandler warnings to prevent MCP stderr noise.
# Context: Ensures PermissionError rotation failures notify once while maintaining fallback behaviour.

from __future__ import annotations

from pathlib import Path

import pytest

from noveler.infrastructure.logging.unified_logger import SafeRotatingFileHandler


def test_permission_error_warning_emitted_once(
    tmp_path: Path, capsys: pytest.CaptureFixture[str]
) -> None:
    """Ensure repeated PermissionError events do not spam stderr output.

    Purpose:
        Confirm that SafeRotatingFileHandler emits the PermissionError warning
        a single time per handler instance while still executing fallback
        rotation logic.

    Args:
        tmp_path (Path): Temporary directory supplied by pytest.
        capsys (pytest.CaptureFixture[str]): Fixture capturing stderr output.

    Returns:
        None: The test asserts on captured stderr content.
    Preconditions:
        pytest must provide functional tmp_path and capsys fixtures.
    Side Effects:
        Creates fallback directories beneath the temporary path.
    """
    log_path = tmp_path / "novel_system.log"
    fallback_dir = tmp_path / "fallback"

    handler = SafeRotatingFileHandler(
        log_path,
        maxBytes=1,
        backupCount=1,
        encoding="utf-8",
        delay=True,
        fallback_dir=fallback_dir,
        fallback_retention_days=0,
    )

    handler._handle_rotation_failure(PermissionError("forced rotation failure"))
    first_err = capsys.readouterr().err

    handler._handle_rotation_failure(PermissionError("repeated failure"))
    second_err = capsys.readouterr().err

    handler.close()

    assert "SafeRotatingFileHandler: PermissionError during rotation" in first_err
    assert "SafeRotatingFileHandler: PermissionError during rotation" not in second_err



"""Application.use_cases.system_diagnosis_use_case
Where: Application use case performing system diagnostics.
What: Runs health checks, aggregates findings, and reports remediation suggestions.
Why: Keeps the platform stable by consolidating diagnostic routines in one workflow.
"""



"""Infrastructure.logging
Where: Infrastructure package exposing logging utilities.
What: Aggregates logging configuration and helper functions for infrastructure services.
Why: Centralises logging setup so infrastructure components share the same patterns.
"""



#!/usr/bin/env python3
"""YAML形式CODEMAPリポジトリ

仕様書: SPEC-CODEMAP-AUTO-UPDATE-001
"""

import shutil
from datetime import datetime, timezone
from pathlib import Path
from typing import Any

import yaml

from noveler.domain.entities.codemap_entity import (
    ArchitectureLayer,
    B20Compliance,
    CircularImportIssue,
    CodeMapEntity,
    CodeMapMetadata,
    QualityPreventionIntegration,
)

# DDD準拠: Infrastructure→Presentation違反を遅延初期化で回避


class _BackupDirectory:
    """Path ラッパー: テストでのメソッド差し替えを容易にする"""

    def __init__(self, path: Path) -> None:
        self._path = path
        self._path.mkdir(exist_ok=True)

    def __truediv__(self, other: object) -> Path:
        return self._path / other

    def __getattr__(self, name: str) -> Any:
        return getattr(self._path, name)

    def glob(self, pattern: str):
        return self._path.glob(pattern)


class YamlCodeMapRepository:
    """YAML形式CODEMAPリポジトリ

    REQ-2.1: YAML読み書き機能
    REQ-2.2: バックアップ・リカバリ機能
    """

    def __init__(self, codemap_path: Path, console_service: Any = None, logger_service: Any = None) -> None:
        """初期化

        Args:
            codemap_path: CODEMAP.yamlファイルパス
            console_service: コンソールサービス（DI注入）
            logger_service: ロガーサービス（DI注入）
        """
        self.codemap_path = codemap_path
        self.backup_dir = _BackupDirectory(codemap_path.parent / ".codemap_backups")
        self._console_service = console_service

        self.logger_service = logger_service
        self.console_service = console_service
    def _get_console(self) -> Any:
        """コンソールサービス取得（DI注入またはフォールバック）"""
        if self._console_service:
            return self._console_service

        # フォールバック: 共通コンソールインスタンス使用
        if not hasattr(self, "_fallback_console"):
            from noveler.presentation.shared.shared_utilities import console

            self._fallback_console = console
        return self._fallback_console

    def load_codemap(self) -> CodeMapEntity | None:
        """CODEMAPを読み込み

        Returns:
            Optional[CodeMapEntity]: CODEMAPエンティティ、失敗時はNone
        """
        try:
            if not self.codemap_path.exists():
                self._get_console().print(f"[yellow]CODEMAP not found: {self.codemap_path}[/yellow]")
                return None

            with self.codemap_path.open(encoding="utf-8") as f:
                data = yaml.safe_load(f)

            return self._parse_yaml_to_entity(data)

        except yaml.YAMLError as e:
            self._get_console().print(f"[red]YAML parsing error: {e}[/red]")
            return None
        except Exception as e:
            self._get_console().print(f"[red]Failed to load CODEMAP: {e}[/red]")
            return None

    def save_codemap(self, codemap: CodeMapEntity) -> bool:
        """CODEMAPを保存

        Args:
            codemap: CODEMAPエンティティ

        Returns:
            bool: 保存成功時True
        """
        try:
            # エンティティを辞書に変換
            data = codemap.to_dict()

            # YAML形式で保存
            with open(self.codemap_path, "w", encoding="utf-8") as f:
                yaml.dump(data, f, default_flow_style=False, allow_unicode=True, sort_keys=False, indent=2)

            self._get_console().print(f"[green]✅ CODEMAP updated: {self.codemap_path}[/green]")
            return True

        except Exception as e:
            self._get_console().print(f"[red]❌ Failed to save CODEMAP: {e}[/red]")
            return False

    def create_backup(self) -> str | None:
        """バックアップ作成

        Returns:
            Optional[str]: バックアップID、失敗時はNone
        """
        try:
            if not self.codemap_path.exists():
                return None

            # マイクロ秒まで含めて連続呼び出しでも重複しないようにする
            timestamp = datetime.now(timezone.utc).strftime("%Y%m%d_%H%M%S_%f")
            backup_id = f"codemap_backup_{timestamp}"
            backup_path = self.backup_dir / f"{backup_id}.yaml"

            shutil.copy2(self.codemap_path, backup_path)

            self._get_console().print(f"[dim]📄 Backup created: {backup_id}[/dim]")
            return backup_id

        except Exception as e:
            self._get_console().print(f"[yellow]⚠️ Failed to create backup: {e}[/yellow]")
            return None

    def restore_from_backup(self, backup_id: str) -> bool:
        """バックアップから復元

        Args:
            backup_id: バックアップID

        Returns:
            bool: 復元成功時True
        """
        try:
            backup_path = self.backup_dir / f"{backup_id}.yaml"

            if not backup_path.exists():
                self._get_console().print(f"[red]Backup not found: {backup_id}[/red]")
                return False

            shutil.copy2(backup_path, self.codemap_path)

            self._get_console().print(f"[yellow]🔄 Restored from backup: {backup_id}[/yellow]")
            return True

        except Exception as e:
            self._get_console().print(f"[red]❌ Failed to restore backup: {e}[/red]")
            return False

    def list_backups(self) -> list[str]:
        """バックアップ一覧取得

        Returns:
            list[str]: バックアップID一覧
        """
        try:
            backups = []
            for backup_file in self.backup_dir.glob("codemap_backup_*.yaml"):
                backup_id = backup_file.stem
                backups.append(backup_id)

            return sorted(backups, reverse=True)  # 新しい順

        except Exception as e:
            self._get_console().print(f"[yellow]⚠️ Failed to list backups: {e}[/yellow]")
            return []

    def cleanup_old_backups(self, keep_count: int = 10) -> int:
        """古いバックアップのクリーンアップ

        Args:
            keep_count: 保持するバックアップ数

        Returns:
            int: 削除したバックアップ数
        """
        try:
            backups = self.list_backups()

            if len(backups) <= keep_count:
                return 0

            deleted_count = 0
            for backup_id in backups[keep_count:]:
                backup_path = self.backup_dir / f"{backup_id}.yaml"
                if backup_path.exists():
                    backup_path.unlink()
                    deleted_count += 1

            if deleted_count > 0:
                self._get_console().print(f"[dim]🗑️ Cleaned up {deleted_count} old backups[/dim]")

            return deleted_count

        except Exception as e:
            self._get_console().print(f"[yellow]⚠️ Failed to cleanup backups: {e}[/yellow]")
            return 0

    def validate_yaml_structure(self) -> list[str]:
        """YAML構造の検証

        Returns:
            list[str]: 検証エラー一覧
        """
        errors: list[Any] = []

        try:
            if not self.codemap_path.exists():
                errors.append("CODEMAP file does not exist")
                return errors

            with self.codemap_path.open(encoding="utf-8") as f:
                data = yaml.safe_load(f)

            # 必須フィールドチェック
            if not isinstance(data, dict):
                errors.append("Root element must be a dictionary")
                return errors

            # project_structureセクション
            if "project_structure" not in data:
                errors.append("Missing 'project_structure' section")
            else:
                ps = data["project_structure"]
                required_fields = ["name", "architecture", "version", "last_updated"]
                for field in required_fields:
                    if field not in ps:
                        errors.append(f"Missing field in project_structure: {field}")

            # circular_import_solutionsセクション
            if "circular_import_solutions" not in data:
                errors.append("Missing 'circular_import_solutions' section")

            # b20_complianceセクション
            if "b20_compliance" not in data:
                errors.append("Missing 'b20_compliance' section")

        except yaml.YAMLError as e:
            errors.append(f"YAML syntax error: {e}")
        except Exception as e:
            errors.append(f"Validation error: {e}")

        return errors

    def _parse_yaml_to_entity(self, data: dict[str, Any]) -> CodeMapEntity:
        """YAML データからエンティティに変換

        Args:
            data: YAML解析結果

        Returns:
            CodeMapEntity: CODEMAPエンティティ
        """
        # project_structure解析
        ps_data: dict[str, Any] = data.get("project_structure", {})

        # last_updatedの解析
        last_updated = ps_data.get("last_updated", "")
        if isinstance(last_updated, str):
            try:
                last_updated = datetime.fromisoformat(last_updated.replace("Z", "+00:00"))
            except ValueError:
                try:
                    last_updated = datetime.strptime(last_updated, "%Y-%m-%d").replace(tzinfo=timezone.utc)
                except ValueError:
                    from noveler.domain.value_objects.project_time import project_now
                    last_updated = project_now().datetime
        elif not isinstance(last_updated, datetime):
            from noveler.domain.value_objects.project_time import project_now
            last_updated = project_now().datetime

        metadata = CodeMapMetadata(
            name=ps_data.get("name", "Unknown Project"),
            architecture=ps_data.get("architecture", "Unknown"),
            version=ps_data.get("version", "1.0.0"),
            last_updated=last_updated,
            commit=ps_data.get("commit"),
        )

        # architecture layers解析
        layers_data: dict[str, Any] = ps_data.get("layers", [])
        architecture_layers = []
        for layer_data in layers_data:
            layer = ArchitectureLayer(
                name=layer_data.get("name", ""),
                path=layer_data.get("path", ""),
                role=layer_data.get("role", ""),
                depends_on=layer_data.get("depends_on", []),
                key_modules=layer_data.get("key_modules", []),
                entry_point=layer_data.get("entry_point"),
            )

            architecture_layers.append(layer)

        # circular_import_solutions解析
        cis_data: dict[str, Any] = data.get("circular_import_solutions", {})
        resolved_issues_data: dict[str, Any] = cis_data.get("resolved_issues", [])
        circular_import_issues = []
        for issue_data in resolved_issues_data:
            issue = CircularImportIssue(
                location=issue_data.get("location", ""),
                issue=issue_data.get("issue", ""),
                solution=issue_data.get("solution", ""),
                status=issue_data.get("status", ""),
                commit=issue_data.get("commit"),
            )

            circular_import_issues.append(issue)

        # b20_compliance解析
        compliance_data: dict[str, Any] = data.get("b20_compliance", {})
        b20_compliance = B20Compliance(
            ddd_layer_separation=compliance_data.get("ddd_layer_separation", {}),
            import_management=compliance_data.get("import_management", {}),
            shared_components=compliance_data.get("shared_components", {}),
        )

        # quality_prevention_integration解析
        quality_data: dict[str, Any] = data.get("quality_prevention_integration", {})
        quality_prevention = QualityPreventionIntegration(
            architecture_linter=quality_data.get("architecture_linter", {}),
            hardcoding_detector=quality_data.get("hardcoding_detector", {}),
            automated_prevention=quality_data.get("automated_prevention", {}),
        )

        return CodeMapEntity(
            metadata=metadata,
            architecture_layers=architecture_layers,
            circular_import_issues=circular_import_issues,
            b20_compliance=b20_compliance,
            quality_prevention=quality_prevention,
        )



#!/usr/bin/env python3
"""YamlDetailedEvaluationRepository インフラ層単体テスト

TDD Red フェーズ: YAML詳細評価リポジトリの失敗テスト
詳細評価セッションと分析結果のYAML永続化機能をテスト
"""

from pathlib import Path

import pytest

from noveler.domain.entities.category_analysis_result import CategoryAnalysisResult
from noveler.domain.entities.detailed_evaluation_session import DetailedEvaluationSession, EvaluationSessionStatus
from noveler.domain.services.detailed_analysis_engine import DetailedAnalysisResult
from noveler.domain.value_objects.a31_evaluation_category import A31EvaluationCategory
from noveler.domain.value_objects.episode_number import EpisodeNumber
from noveler.domain.value_objects.improvement_suggestion import ImprovementSuggestion
from noveler.domain.value_objects.line_specific_feedback import IssueSeverity, IssueType, LineSpecificFeedback
from noveler.infrastructure.repositories.yaml_detailed_evaluation_repository import (
    YamlDetailedEvaluationRepository,
)


@pytest.mark.spec("SPEC-A31-DET-001")
class TestYamlDetailedEvaluationRepository:
    """YamlDetailedEvaluationRepository インフラ層テスト"""

    @pytest.fixture(autouse=True)
    def setup_method(self, tmp_path: Path) -> None:
        """テストセットアップ"""
        self.temp_dir = str(tmp_path)
        self.repository = YamlDetailedEvaluationRepository(self.temp_dir)

        # テスト用エリアルデータ
        self.test_project_name = "テストプロジェクト"
        self.test_episode_number = EpisodeNumber(1)
        self.test_episode_content = """# 第001話 テストエピソード

「こんにちは」と彼女は言った。

俺は驚いた。だった。だった。

彼女の笑顔が美しかった。"""

    def test_save_and_load_evaluation_session_successfully(self) -> None:
        """評価セッションの保存と読み込みが正常に動作する"""
        # Given
        session = DetailedEvaluationSession.create(
            project_name=self.test_project_name,
            episode_number=self.test_episode_number,
            episode_content=self.test_episode_content,
        )

        session.start_evaluation()

        # カテゴリ分析結果を追加
        category_result = CategoryAnalysisResult.create(
            category=A31EvaluationCategory.STYLE_CONSISTENCY,
            score=75.0,
            issues_found=["文末単調性問題"],
            suggestions=["文末バリエーションを増やす"],
        )

        session.add_category_analysis(category_result)

        # When - 保存
        self.repository.save_evaluation_session(session)

        # Then - 読み込み確認
        loaded_session = self.repository.get_evaluation_session(self.test_project_name, self.test_episode_number)

        assert loaded_session is not None
        assert loaded_session.project_name == self.test_project_name
        assert loaded_session.episode_number == self.test_episode_number
        assert loaded_session.episode_content == self.test_episode_content
        assert loaded_session.status == EvaluationSessionStatus.IN_PROGRESS
        assert len(loaded_session.category_analyses) == 1
        assert loaded_session.category_analyses[0].category == A31EvaluationCategory.STYLE_CONSISTENCY

    def test_save_and_load_analysis_result_with_line_feedbacks(self) -> None:
        """行別フィードバック付き分析結果の保存と読み込み"""
        # Given
        session_id = "test-session-123"

        # カテゴリ分析結果作成
        category_result = CategoryAnalysisResult.create(
            category=A31EvaluationCategory.CONTENT_BALANCE,
            score=82.5,
            issues_found=["会話バランス問題"],
            suggestions=["地の文を追加"],
        )

        # 行別フィードバック作成
        improvement_suggestion = ImprovementSuggestion.create(
            content="この箇所に状況説明を追加してください",
            suggestion_type="content_enhancement",
            confidence=0.9,
            fix_example="「こんにちは」と彼女は笑顔で言った。朝の光が彼女の髪を照らしている。",
        )

        line_feedback = LineSpecificFeedback.create(
            line_number=3,
            original_text="「こんにちは」と彼女は言った。",
            issue_type=IssueType.CONTENT_BALANCE.value,
            severity=IssueSeverity.MINOR.value,
            suggestion=improvement_suggestion.content,
            confidence=0.9,
            context_lines=["# 第001話 テストエピソード", "", "「こんにちは」と彼女は言った。", "", "俺は驚いた。"],
        )

        # 分析結果作成
        analysis_result = DetailedAnalysisResult(
            session_id=session_id,
            overall_score=78.2,
            category_results=[category_result],
            line_feedbacks=[line_feedback],
            confidence_score=0.85,
            analysis_summary={"total_issues": 1, "total_suggestions": 1, "analysis_depth": "comprehensive"},
        )

        # When - 保存
        self.repository.save_analysis_result(analysis_result)

        # Then - 読み込み確認
        loaded_result = self.repository.get_analysis_result(session_id)

        assert loaded_result is not None
        assert loaded_result.session_id == session_id
        assert loaded_result.overall_score == 78.2
        assert loaded_result.confidence_score == 0.85
        assert len(loaded_result.category_results) == 1
        assert len(loaded_result.line_feedbacks) == 1

        # カテゴリ結果詳細確認
        loaded_category = loaded_result.category_results[0]
        assert loaded_category.category == A31EvaluationCategory.CONTENT_BALANCE
        assert loaded_category.score == 82.5

        # 行フィードバック詳細確認
        loaded_feedback = loaded_result.line_feedbacks[0]
        assert loaded_feedback.line_number == 3
        assert loaded_feedback.issue_type == IssueType.CONTENT_BALANCE
        assert loaded_feedback.severity == IssueSeverity.MINOR

    def test_list_evaluation_sessions_returns_sorted_episodes(self) -> None:
        """評価セッション一覧がエピソード番号順でソートされて返される"""
        # Given - 複数のエピソードセッションを作成
        episodes = [3, 1, 5, 2]
        sessions = []

        for ep_num in episodes:
            session = DetailedEvaluationSession.create(
                project_name=self.test_project_name,
                episode_number=EpisodeNumber(ep_num),
                episode_content=f"エピソード{ep_num}の内容",
            )

            sessions.append(session)
            self.repository.save_evaluation_session(session)

        # When
        loaded_sessions = self.repository.list_evaluation_sessions(self.test_project_name)

        # Then
        assert len(loaded_sessions) == 4
        episode_numbers = [session.episode_number.value for session in loaded_sessions]
        assert episode_numbers == [1, 2, 3, 5]  # ソートされている

    def test_delete_evaluation_session_removes_files(self) -> None:
        """評価セッション削除でファイルが削除される"""
        # Given
        session = DetailedEvaluationSession.create(
            project_name=self.test_project_name,
            episode_number=self.test_episode_number,
            episode_content=self.test_episode_content,
        )

        self.repository.save_evaluation_session(session)

        # 存在確認
        assert self.repository.exists_evaluation_session(self.test_project_name, self.test_episode_number) is True

        # When
        result = self.repository.delete_evaluation_session(self.test_project_name, self.test_episode_number)

        # Then
        assert result is True
        assert self.repository.exists_evaluation_session(self.test_project_name, self.test_episode_number) is False

        # 読み込み不可確認
        loaded_session = self.repository.get_evaluation_session(self.test_project_name, self.test_episode_number)

        assert loaded_session is None

    def test_sanitize_filename_handles_unsafe_characters(self) -> None:
        """ファイル名の危険文字がサニタイズされる"""
        # Given
        unsafe_project_name = 'プロジェクト<>:"/\\|?*'

        # When
        safe_filename = self.repository._sanitize_filename(unsafe_project_name)

        # Then
        assert safe_filename == "プロジェクト_________"
        assert not any(char in safe_filename for char in '<>:"/\\|?*')

    def test_handle_nonexistent_files_gracefully(self) -> None:
        """存在しないファイルの操作が適切に処理される"""
        # Given - 存在しないセッション
        nonexistent_episode = EpisodeNumber(999)

        # When & Then - 存在しないセッションの取得
        session = self.repository.get_evaluation_session(self.test_project_name, nonexistent_episode)

        assert session is None

        # When & Then - 存在しないセッションの削除
        result = self.repository.delete_evaluation_session(self.test_project_name, nonexistent_episode)

        assert result is False

        # When & Then - 存在しない分析結果の取得
        analysis_result = self.repository.get_analysis_result("nonexistent-session")
        assert analysis_result is None

    def test_yaml_serialization_preserves_unicode_content(self) -> None:
        """YAML シリアライゼーションでUnicode内容が保持される"""
        # Given - 日本語を含むセッション
        japanese_content = """# 第001話 桜咲く季節

「お疲れさまでした」と彼女は深々と頭を下げた。

私の心は温かくなった。これこそが求めていた瞬間だった。"""

        session = DetailedEvaluationSession.create(
            project_name="日本語プロジェクト", episode_number=EpisodeNumber(1), episode_content=japanese_content
        )

        # When
        self.repository.save_evaluation_session(session)
        loaded_session = self.repository.get_evaluation_session("日本語プロジェクト", EpisodeNumber(1))

        # Then
        assert loaded_session is not None
        assert loaded_session.episode_content == japanese_content
        assert loaded_session.project_name == "日本語プロジェクト"

    def test_repository_directory_creation(self, tmp_path: Path) -> None:
        """リポジトリの必要ディレクトリが自動作成される"""
        # Given - 新しい一時ディレクトリ
        new_temp_dir = tmp_path / "new_repo_location"

        # When - 新しいリポジトリ作成
        YamlDetailedEvaluationRepository(str(new_temp_dir))

        # Then - ディレクトリが存在することを確認
        assert (new_temp_dir / "sessions").exists()
        assert (new_temp_dir / "results").exists()

    def test_concurrent_access_safety(self) -> None:
        """同時アクセス時の安全性確認"""
        # Given
        session1 = DetailedEvaluationSession.create(
            project_name=self.test_project_name, episode_number=EpisodeNumber(1), episode_content="セッション1"
        )

        session2 = DetailedEvaluationSession.create(
            project_name=self.test_project_name, episode_number=EpisodeNumber(2), episode_content="セッション2"
        )

        # When - 同時保存
        self.repository.save_evaluation_session(session1)
        self.repository.save_evaluation_session(session2)

        # Then - 両方とも正常に保存・読み込み可能
        loaded1 = self.repository.get_evaluation_session(self.test_project_name, EpisodeNumber(1))

        loaded2 = self.repository.get_evaluation_session(self.test_project_name, EpisodeNumber(2))

        assert loaded1 is not None
        assert loaded2 is not None
        assert loaded1.episode_content == "セッション1"
        assert loaded2.episode_content == "セッション2"



#!/usr/bin/env python3
"""シーン管理DTO群

シーン管理ユースケースのリクエスト・レスポンスオブジェクト定義。
アプリケーション層のデータ転送オブジェクト(DTO)として分離。
"""

from dataclasses import dataclass, field
from pathlib import Path

from noveler.domain.entities.scene_management_entities import SceneCategory, SceneInfo, ValidationIssue


@dataclass
class SceneInitRequest:
    """シーン初期化リクエスト"""

    project_name: str
    project_directory: str


@dataclass
class SceneInitResponse:
    """シーン初期化レスポンス"""

    success: bool
    scene_file_path: Path | None = None
    message: str = ""


@dataclass
class SceneAddRequest:
    """シーン追加リクエスト"""

    project_name: str
    project_directory: str
    category: SceneCategory
    scene_id: str
    title: str
    description: str
    episodes: list[int] = field(default_factory=list)
    sensory_details: dict[str, str] = field(default_factory=dict)


@dataclass
class SceneAddResponse:
    """シーン追加レスポンス"""

    success: bool
    scene_id: str
    category: SceneCategory
    message: str = ""


@dataclass
class SceneListRequest:
    """シーン一覧リクエスト"""

    project_name: str
    project_directory: str
    category: SceneCategory | None = None


@dataclass
class SceneListResponse:
    """シーン一覧レスポンス"""

    success: bool
    scenes: list[SceneInfo] = field(default_factory=list)
    total_by_category: dict[SceneCategory, int] = field(default_factory=dict)
    message: str = ""


@dataclass
class SceneShowRequest:
    """シーン詳細リクエスト"""

    project_name: str
    project_directory: str
    category: SceneCategory
    scene_id: str


@dataclass
class SceneShowResponse:
    """シーン詳細レスポンス"""

    success: bool
    scene: SceneInfo | None = None
    message: str = ""


@dataclass
class SceneChecklistRequest:
    """シーンチェックリストリクエスト"""

    project_name: str
    project_directory: str
    output_format: str = "markdown"
    output_path: str | None = None


@dataclass
class SceneChecklistResponse:
    """シーンチェックリストレスポンス"""

    success: bool
    checklist_content: str = ""
    total_scenes: int = 0
    completed_scenes: int = 0
    output_path: Path | None = None
    message: str = ""


@dataclass
class SceneValidateRequest:
    """シーン検証リクエスト"""

    project_name: str
    project_directory: str


@dataclass
class SceneValidateResponse:
    """シーン検証レスポンス"""

    is_valid: bool
    issues: list[ValidationIssue] = field(default_factory=list)
    error_count: int = 0
    warning_count: int = 0
    message: str = ""



# File: src/noveler/infrastructure/utils/infra_path.py
# Purpose: Provide infrastructure-safe path service access without direct
#          presentation layer dependencies.
# Context: B20/DDD compliance - Infrastructure layer should use factories
#          from application layer for path service access

"""Infrastructure-safe path service access utilities.

This module provides path service access for Infrastructure layer services
without violating DDD layering principles. It delegates to application
layer factories which manage dependency injection properly.

Usage:
    from noveler.infrastructure.utils.infra_path import get_path_service

    path_service = get_path_service()
    project_root = path_service.project_root
    manuscript_dir = path_service.get_manuscript_dir()
"""

from __future__ import annotations

from pathlib import Path
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from noveler.domain.services.path_service import PathService

__all__ = ["get_path_service", "get_project_root"]


def get_path_service(project_root: str | Path | None = None) -> PathService:
    """Return a PathService instance for infrastructure layer services.

    Delegates to application layer factory which handles proper
    dependency injection without violating DDD boundaries.

    Args:
        project_root: Optional project root path. If None, uses current directory.

    Returns:
        PathService instance configured for the given project root
    """
    from noveler.infrastructure.factories.path_service_factory import create_path_service

    return create_path_service(project_root)


def get_project_root(project_root: str | Path | None = None) -> Path:
    """Return the project root path.

    Convenience function for infrastructure services that only need
    the project root without full PathService interface.

    Args:
        project_root: Optional project root path. If None, uses current directory.

    Returns:
        Path to project root
    """
    return get_path_service(project_root).project_root



#!/usr/bin/env python3
"""Archive disabled configs and backups into a standardized folder.

Usage:
  python scripts/tools/archive_disabled_configs.py \
    --project-root . \
    --dest archive/disabled_configs \
    --patterns "*.disabled.json,*.backup" \
    --dry-run

- Moves files matching the patterns into `{dest}/{YYYYMMDD}/` while
  preserving relative directory structure.
- Skips common directories (e.g., .git, dist, .mcp cache).
"""
from __future__ import annotations

import argparse
import datetime as _dt
import fnmatch
import shutil
from pathlib import Path
import os
from collections.abc import Iterable


SKIP_DIRS = {".git", "__pycache__", ".mcp", ".tmp", "dist", "build"}


def iter_files(root: Path, patterns: list[str]) -> Iterable[Path]:
    """Robust file iterator that skips problematic/broken directories."""
    for dirpath, dirnames, filenames in os.walk(root, topdown=True):
        # prune skip dirs
        dirnames[:] = [d for d in dirnames if d not in SKIP_DIRS]
        # also prune hidden heavy dirs quickly
        dirnames[:] = [d for d in dirnames if not d.startswith(".") or d == ".noveler"]
        for name in filenames:
            try:
                if any(fnmatch.fnmatch(name, pat) for pat in patterns):
                    p = Path(dirpath) / name
                    yield p
            except Exception:
                # ignore transient errors
                continue


def main() -> int:
    ap = argparse.ArgumentParser()
    ap.add_argument("--project-root", type=Path, default=Path("."))
    ap.add_argument(
        "--patterns",
        type=str,
        default="*.disabled.json,*.backup",
        help="Comma-separated glob patterns",
    )
    ap.add_argument(
        "--dest",
        type=Path,
        default=Path("archive/disabled_configs"),
        help="Destination base directory",
    )
    ap.add_argument("--dry-run", action="store_true")
    args = ap.parse_args()

    root = args.project_root.resolve()
    patterns = [s.strip() for s in str(args.patterns).split(",") if s.strip()]
    date_dir = _dt.datetime.now().strftime("%Y%m%d")
    base_dest = (args.dest / date_dir).resolve()
    base_dest.mkdir(parents=True, exist_ok=True)

    moved = 0
    for src in iter_files(root, patterns):
        rel = src.relative_to(root)
        dest_path = base_dest / rel
        dest_path.parent.mkdir(parents=True, exist_ok=True)
        if args.dry_run:
            print(f"[DRY-RUN] move {src} -> {dest_path}")
        else:
            shutil.move(str(src), str(dest_path))
            print(f"moved {src} -> {dest_path}")
            moved += 1

    print(f"✅ Done. candidates={moved if not args.dry_run else 'dry-run'} dest={base_dest}")
    return 0


if __name__ == "__main__":
    raise SystemExit(main())



#!/usr/bin/env python3
"""B20開発段階Value Objectのテスト

仕様書: B20開発作業指示書準拠
"""

import pytest

from noveler.domain.value_objects.b20_development_stage import B20DevelopmentStage, DevelopmentStage, StageRequirement

pytestmark = pytest.mark.vo_smoke



class TestB20DevelopmentStage:
    """B20開発段階Value Objectのテスト"""

    @pytest.mark.spec("SPEC-B20_DEVELOPMENT_STAGE-CREATE_SPECIFICATION")
    def test_create_specification_stage(self):
        """仕様書作成段階の作成テスト"""
        stage = B20DevelopmentStage.create_specification_stage()

        assert stage.current_stage == DevelopmentStage.SPECIFICATION_REQUIRED
        assert len(stage.completed_requirements) == 0
        assert StageRequirement.SPEC_DOCUMENT_EXISTS in stage.pending_requirements
        assert "仕様書作成必須段階" in stage.stage_metadata["description"]

    @pytest.mark.spec("SPEC-B20_DEVELOPMENT_STAGE-CREATE_CODEMAP_CHECK")
    def test_create_codemap_check_stage(self):
        """CODEMAP確認段階の作成テスト"""
        stage = B20DevelopmentStage.create_codemap_check_stage()

        assert stage.current_stage == DevelopmentStage.CODEMAP_CHECK_REQUIRED
        assert StageRequirement.SPEC_DOCUMENT_EXISTS in stage.completed_requirements
        assert StageRequirement.CODEMAP_UPDATED in stage.pending_requirements
        assert StageRequirement.IMPORT_CONFLICTS_CHECKED in stage.pending_requirements

    @pytest.mark.spec("SPEC-B20_DEVELOPMENT_STAGE-CREATE_IMPLEMENTATIO")
    def test_create_implementation_stage(self):
        """実装許可段階の作成テスト"""
        stage = B20DevelopmentStage.create_implementation_stage()

        assert stage.current_stage == DevelopmentStage.IMPLEMENTATION_ALLOWED
        assert StageRequirement.SPEC_DOCUMENT_EXISTS in stage.completed_requirements
        assert StageRequirement.CODEMAP_UPDATED in stage.completed_requirements
        assert StageRequirement.TEST_IMPLEMENTATION_COMPLETED in stage.pending_requirements

    @pytest.mark.spec("SPEC-B20_DEVELOPMENT_STAGE-ADVANCE_TO_NEXT_STAG")
    def test_advance_to_next_stage(self):
        """次段階への進行テスト"""
        # 仕様書段階から開始
        stage = B20DevelopmentStage.create_specification_stage()

        # 仕様書完了で次段階へ進行
        next_stage = stage.advance_to_next_stage(StageRequirement.SPEC_DOCUMENT_EXISTS)

        assert next_stage.current_stage == DevelopmentStage.CODEMAP_CHECK_REQUIRED
        assert StageRequirement.SPEC_DOCUMENT_EXISTS in next_stage.completed_requirements
        assert StageRequirement.SPEC_DOCUMENT_EXISTS not in next_stage.pending_requirements

    @pytest.mark.spec("SPEC-B20_DEVELOPMENT_STAGE-ADVANCE_TO_NEXT_STAG")
    def test_advance_to_next_stage_invalid_requirement(self):
        """無効な要件での段階進行テスト"""
        stage = B20DevelopmentStage.create_specification_stage()

        # 未ペンディングの要件で進行を試行
        with pytest.raises(ValueError, match="要件が未ペンディング"):
            stage.advance_to_next_stage(StageRequirement.CODEMAP_UPDATED)

    @pytest.mark.spec("SPEC-B20_DEVELOPMENT_STAGE-GET_NEXT_REQUIRED_AC")
    def test_get_next_required_actions(self):
        """次に必要な作業項目の取得テスト"""
        stage = B20DevelopmentStage.create_specification_stage()
        actions = stage.get_next_required_actions()

        assert len(actions) > 0
        assert "仕様書の作成（specs/ディレクトリ）" in actions

    @pytest.mark.spec("SPEC-B20_DEVELOPMENT_STAGE-IS_IMPLEMENTATION_AL")
    def test_is_implementation_allowed(self):
        """実装許可状態の確認テスト"""
        # 仕様書段階では実装不可
        spec_stage = B20DevelopmentStage.create_specification_stage()
        assert not spec_stage.is_implementation_allowed()

        # 実装段階では実装可能
        impl_stage = B20DevelopmentStage.create_implementation_stage()
        assert impl_stage.is_implementation_allowed()

    @pytest.mark.spec("SPEC-B20_DEVELOPMENT_STAGE-IS_COMMIT_ALLOWED")
    def test_is_commit_allowed(self):
        """コミット許可状態の確認テスト"""
        # 実装段階ではコミット不可
        impl_stage = B20DevelopmentStage.create_implementation_stage()
        assert not impl_stage.is_commit_allowed()

        # 全要件完了でコミット許可段階になる
        all_completed = frozenset(
            [
                StageRequirement.SPEC_DOCUMENT_EXISTS,
                StageRequirement.CODEMAP_UPDATED,
                StageRequirement.IMPORT_CONFLICTS_CHECKED,
                StageRequirement.DDD_LAYER_PLACEMENT_VALIDATED,
                StageRequirement.EXISTING_IMPLEMENTATION_CHECKED,
                StageRequirement.TEST_IMPLEMENTATION_COMPLETED,
            ]
        )

        commit_stage = B20DevelopmentStage(
            current_stage=DevelopmentStage.COMMIT_ALLOWED,
            completed_requirements=all_completed,
            pending_requirements=frozenset(),
            stage_metadata={"test": True},
        )

        assert commit_stage.is_commit_allowed()

    @pytest.mark.spec("SPEC-B20_DEVELOPMENT_STAGE-GET_COMPLETION_PERCE")
    def test_get_completion_percentage(self):
        """完了率の計算テスト"""
        # 仕様書段階（0%完了）
        spec_stage = B20DevelopmentStage.create_specification_stage()
        assert spec_stage.get_completion_percentage() == 0.0

        # 1つの要件完了（約16.7%完了）
        one_completed = frozenset([StageRequirement.SPEC_DOCUMENT_EXISTS])
        partial_stage = B20DevelopmentStage(
            current_stage=DevelopmentStage.CODEMAP_CHECK_REQUIRED,
            completed_requirements=one_completed,
            pending_requirements=frozenset(
                [
                    StageRequirement.CODEMAP_UPDATED,
                    StageRequirement.IMPORT_CONFLICTS_CHECKED,
                    StageRequirement.DDD_LAYER_PLACEMENT_VALIDATED,
                    StageRequirement.EXISTING_IMPLEMENTATION_CHECKED,
                    StageRequirement.TEST_IMPLEMENTATION_COMPLETED,
                ]
            ),
            stage_metadata={"test": True},
        )

        completion = partial_stage.get_completion_percentage()
        assert 16.0 <= completion <= 17.0  # 約16.7%

    @pytest.mark.spec("SPEC-B20_DEVELOPMENT_STAGE-GET_STAGE_DESCRIPTIO")
    def test_get_stage_description(self):
        """段階説明の取得テスト"""
        stages_and_descriptions = [
            (DevelopmentStage.SPECIFICATION_REQUIRED, "仕様書作成が必要です"),
            (DevelopmentStage.CODEMAP_CHECK_REQUIRED, "CODEMAPの確認・更新が必要です"),
            (DevelopmentStage.IMPLEMENTATION_ALLOWED, "実装を開始できます"),
            (DevelopmentStage.COMMIT_ALLOWED, "コミットが許可されています"),
        ]

        for stage_enum, expected_desc in stages_and_descriptions:
            stage = B20DevelopmentStage(
                current_stage=stage_enum,
                completed_requirements=frozenset(),
                pending_requirements=frozenset(),
                stage_metadata={"test": True},
            )

            assert stage.get_stage_description() == expected_desc

    @pytest.mark.spec("SPEC-B20_DEVELOPMENT_STAGE-INITIALIZATION_VALID")
    def test_initialization_validation(self):
        """初期化時の検証テスト"""
        # 要件の重複エラー
        with pytest.raises(ValueError, match="要件の重複検出"):
            B20DevelopmentStage(
                current_stage=DevelopmentStage.SPECIFICATION_REQUIRED,
                completed_requirements=frozenset([StageRequirement.SPEC_DOCUMENT_EXISTS]),
                pending_requirements=frozenset([StageRequirement.SPEC_DOCUMENT_EXISTS]),  # 重複
                stage_metadata={"test": True},
            )

    @pytest.mark.spec("SPEC-B20_DEVELOPMENT_STAGE-STAGE_TRANSITION_LOG")
    def test_stage_transition_logic(self):
        """段階遷移ロジックのテスト"""
        # 仕様書 -> CODEMAP確認 -> 実装許可 -> コミット許可の流れをテスト
        stage = B20DevelopmentStage.create_specification_stage()

        # Step 1: 仕様書完了
        stage = stage.advance_to_next_stage(StageRequirement.SPEC_DOCUMENT_EXISTS)
        assert stage.current_stage == DevelopmentStage.CODEMAP_CHECK_REQUIRED

        # Step 2: CODEMAP関連の要件を順次完了
        requirements_to_complete = [
            StageRequirement.CODEMAP_UPDATED,
            StageRequirement.IMPORT_CONFLICTS_CHECKED,
            StageRequirement.DDD_LAYER_PLACEMENT_VALIDATED,
            StageRequirement.EXISTING_IMPLEMENTATION_CHECKED,
        ]

        for requirement in requirements_to_complete:
            stage = stage.advance_to_next_stage(requirement)

        assert stage.current_stage == DevelopmentStage.IMPLEMENTATION_ALLOWED

        # Step 3: テスト実装完了
        stage = stage.advance_to_next_stage(StageRequirement.TEST_IMPLEMENTATION_COMPLETED)
        assert stage.current_stage == DevelopmentStage.COMMIT_ALLOWED

    @pytest.mark.spec("SPEC-B20_DEVELOPMENT_STAGE-IMMUTABILITY")
    def test_immutability(self):
        """Value Objectの不変性テスト"""
        stage = B20DevelopmentStage.create_specification_stage()
        original_completed = stage.completed_requirements

        # 新しい段階を作成
        new_stage = stage.advance_to_next_stage(StageRequirement.SPEC_DOCUMENT_EXISTS)

        # 元のオブジェクトは変更されていないことを確認
        assert stage.completed_requirements == original_completed
        assert stage.current_stage == DevelopmentStage.SPECIFICATION_REQUIRED

        # 新しいオブジェクトは変更されていることを確認
        assert new_stage.completed_requirements != original_completed
        assert new_stage.current_stage != DevelopmentStage.SPECIFICATION_REQUIRED
